from ase.md.analysis import DiffusionCoefficient
from ase.atoms import Atoms
from ase.units import fs as fs_conversion

eps = 1e-10
# Creating simple trajectories
# Textbook case. The displacement coefficient should be 0.5 A^2 / fs except for the final molecule

###### He atom

he = Atoms('He', positions=[(0, 0, 0)])
traj_he = [he.copy() for i in range(2)]
traj_he[1].set_positions([(1, 1, 1)])

timestep = 1 * fs_conversion #fs

dc_he = DiffusionCoefficient(traj_he, timestep)
dc_he.calculate(ignore_n_images=0, number_of_segments=1)
ans = dc_he.get_diffusion_coefficients()[0][0]
# Answer in \AA^2/<ASE time unit>
ans_orig = 5.0e-01 / fs_conversion
#dc_he.print_data()

assert(abs(ans - ans_orig) < eps)

###### CO molecule

co = Atoms('CO', positions=[(0, 0, 0), (0, 0, 1)])
traj_co = [co.copy() for i in range(2)]
traj_co[1].set_positions([(-1, -1, -1), (-1, -1, 0)])

dc_co = DiffusionCoefficient(traj_co, timestep, molecule=False)
dc_co.calculate(ignore_n_images=0, number_of_segments=1)
ans = dc_co.get_diffusion_coefficients()[0][0]
#dc_co.print_data()

assert(abs(ans - ans_orig) < eps)

for index in range(2):
    dc_co = DiffusionCoefficient(traj_co, timestep, atom_indices=[index], molecule=False)
    dc_co.calculate() 
    ans = dc_co.get_diffusion_coefficients()[0][0]
    assert(abs(ans - ans_orig) < eps)

dc_co = DiffusionCoefficient(traj_co, timestep, molecule=True)
dc_co.calculate(ignore_n_images=0, number_of_segments=1)
ans = dc_co.get_diffusion_coefficients()[0][0]
#dc_co.print_data()

assert(abs(ans - ans_orig) < eps)


from ase.io import write
from ase.ga import get_raw_score
from ase.ga.data import DataConnection
from ase.ga.population import Population
from ase.ga.utilities import closest_distances_generator
from ase.ga.bulk_utilities import CellBounds
from ase.ga.ofp_comparator import OFPComparator
from ase.ga.offspring_creator import OperationSelector
from ase.ga.bulk_mutations import StrainMutation, SoftMutation
from ase.ga.bulk_crossovers import CutAndSplicePairing
from ga_bulk_relax import relax


# Connect to the database and retrieve some information
da = DataConnection('gadb.db')
atom_numbers_to_optimize = da.get_atom_numbers_to_optimize()
n_to_optimize = len(atom_numbers_to_optimize)

# Use Oganov's fingerprint functions to decide whether
# two structures are identical or not
comp = OFPComparator(n_top=n_to_optimize, dE=1.0,
                     cos_dist_max=1e-3, rcut=10., binwidth=0.05,
                     pbc=[True, True, True], sigma=0.05, nsigma=4,
                     recalculate=False)

# Define the cell and interatomic distance bounds
# that the candidates must obey
blmin = closest_distances_generator(atom_numbers_to_optimize, 0.5)

cellbounds = CellBounds(bounds={'phi': [20, 160],
                                'chi': [20, 160],
                                'psi': [20, 160],
                                'a': [2, 60], 'b': [2, 60], 'c': [2, 60]})

# Define a pairing operator with 100% (0%) chance that the first
# (second) parent will be randomly translated, and with each parent
# contributing to at least 15% of the child's scaled coordinates
pairing = CutAndSplicePairing(blmin, p1=1., p2=0., minfrac=0.15,
                              cellbounds=cellbounds, use_tags=False)

# Define a strain mutation with a typical standard deviation of 0.7
# for the strain matrix elements (drawn from a normal distribution)
strainmut = StrainMutation(blmin, stddev=0.7, cellbounds=cellbounds,
                           use_tags=False)

# Define a soft mutation; we need to provide a dictionary with
# (typically rather short) minimal interatomic distances which
# is used to determine when to stop displacing the atoms along
# the chosen mode. The minimal and maximal single-atom displacement
# distances (in Angstrom) for a valid mutation are provided via
# the 'bounds' keyword argument.
blmin_soft = closest_distances_generator(atom_numbers_to_optimize, 0.1)
softmut = SoftMutation(blmin_soft, bounds=[2., 5.], use_tags=False)
# By default, the operator will update a "used_modes.json" file
# after every mutation, listing which modes have been used so far
# for each structure in the database. The mode indices start at 3
# as the three lowest frequency modes are translational modes.

# Set up the relative probabilities for the different operators
operators = OperationSelector([4., 3., 3.],
                              [pairing, softmut, strainmut])

# Relax the initial candidates
while da.get_number_of_unrelaxed_candidates() > 0:
    a = da.get_an_unrelaxed_candidate()

    relax(a, cellbounds=cellbounds)
    da.add_relaxed_step(a)

    cell = a.get_cell()
    if not cellbounds.is_within_bounds(cell):
        da.kill_candidate(a.info['confid'])

# Initialize the population
population_size = 20
population = Population(data_connection=da,
                        population_size=population_size,
                        comparator=comp,
                        logfile='log.txt',
                        use_extinct=True)

# Update the scaling volume used in some operators
# based on a number of the best candidates
current_pop = population.get_current_population()
strainmut.update_scaling_volume(current_pop, w_adapt=0.5, n_adapt=4)
pairing.update_scaling_volume(current_pop, w_adapt=0.5, n_adapt=4)

# Test n_to_test new candidates; in this example we need
# only few GA iterations as the global minimum (FCC Ag)
# is very easily found (typically already after relaxation
# of the initial random structures).
n_to_test = 50

for step in range(n_to_test):
    print('Now starting configuration number {0}'.format(step))

    # Create a new candidate
    a3 = None
    while a3 is None:
        a1, a2 = population.get_two_candidates()
        a3, desc = operators.get_new_individual([a1, a2])

    # Save the unrelaxed candidate
    da.add_unrelaxed_candidate(a3, description=desc)

    # Relax the new candidate and save it
    relax(a3, cellbounds=cellbounds)
    da.add_relaxed_step(a3)

    # If the relaxation has changed the cell parameters
    # beyond the bounds we disregard it in the population
    cell = a3.get_cell()
    if not cellbounds.is_within_bounds(cell):
        da.kill_candidate(a3.info['confid'])

    # Update the population
    population.update()

    if step % 10 == 0:
        # Update the scaling volumes of the strain mutation
        # and the pairing operator based on the current
        # best structures contained in the population
        current_pop = population.get_current_population()
        strainmut.update_scaling_volume(current_pop, w_adapt=0.5,
                                        n_adapt=4)
        pairing.update_scaling_volume(current_pop, w_adapt=0.5, n_adapt=4)
        write('current_population.traj', current_pop)

print('GA finished after step %d' % step)
hiscore = get_raw_score(current_pop[0])
print('Highest raw score = %8.4f eV' % hiscore)

all_candidates = da.get_all_relaxed_candidates()
write('all_candidates.traj', all_candidates)

current_pop = population.get_current_population()
write('current_population.traj', current_pop)



import numpy as np
from ase.md.langevin import Langevin
from ase.calculators.mixing import MixedCalculator


class SwitchLangevin(Langevin):
    """
    MD class for carrying out thermodynamic integration between two
    hamiltonians

    Read more at https://en.wikipedia.org/wiki/Thermodynamic_integration

    The integration path is lambda 0 ---> 1 , with the switching hamiltonian
    H(lambda) = (1-lambda) * H1 + lambda * H2

    Attributes
    ----------
    path_data : numpy array
        col 1 (md-step), col 2 (lambda), col 3 (energy H1), col 4 (energy H2)

    Parameters
    ----------
    atoms : ASE Atoms object
        Atoms object for which MD will be run
    calc1 : ASE calculator object
        Calculator correpsonding to first Hamiltonian
    calc2 : ASE calculator object
        Calculator corresponding to second Hamiltonian
    dt : float
        Timestep for MD simulation
    T : float
        Temperature
    friction : float
        Friction for langevin dynamics
    n_eq : int
        Number of equilibration steps
    n_switch : int
        Number of switching steps
    """

    def __init__(self, atoms, calc1, calc2, dt, T, friction, n_eq, n_switch,
                 **langevin_kwargs):
        super().__init__(atoms, dt, T, friction, **langevin_kwargs)
        self.n_eq = n_eq
        self.n_switch = n_switch
        self.lam = 0.0
        calc = MixedCalculator(calc1, calc2, weight1=1.0, weight2=0.0)
        self.atoms.set_calculator(calc)

        self.path_data = []

    def run(self):
        """ Run the MD switching simulation """
        forces = self.atoms.get_forces(md=True)

        # run equilibration with calc1
        for _ in range(self.n_eq):
            forces = self.step(forces)
            self.nsteps += 1
            self.call_observers()

        # run switch from calc1 to calc2
        self.path_data.append([0, self.lam, *self.atoms.calc.get_energy_contributions(self.atoms)])
        for step in range(1, self.n_switch):
            # update calculator
            self.lam = get_lambda(step, self.n_switch)
            self.atoms.calc.set_weights(1 - self.lam, self.lam)

            # carry out md step
            forces = self.step(forces)
            self.nsteps += 1

            # collect data
            self.call_observers()
            self.path_data.append([step, self.lam, *self.atoms.calc.get_energy_contributions(self.atoms)])

        self.path_data = np.array(self.path_data)

    def get_free_energy_difference(self):
        """ Return the free energy difference between calc2 and calc1, by
        integrating dH/dlam along the switching path

        Returns
        -------
        float
            Free energy difference, F2 - F1
        """
        if len(self.path_data) == 0:
            raise ValueError('No free energy data found.')

        lambdas = self.path_data[:, 1]
        U1 = self.path_data[:, 2]
        U2 = self.path_data[:, 3]
        delta_F = np.trapz(U2 - U1, lambdas)
        return delta_F


def get_lambda(step, n_switch):
    """ Return lambda value along the switching path """
    assert step >= 0 and step <= n_switch
    t = step / (n_switch - 1)
    return t**5 * (70 * t**4 - 315 * t**3 + 540 * t**2 - 420 * t + 126)



from ase.db import connect

refs = connect('refs.db')
db = connect('ads.db')

for row in db.select():
    ea = (row.energy -
          refs.get(formula=row.ads).energy -
          refs.get(layers=row.layers, surf=row.surf).energy)
    h = row.positions[-1, 2] - row.positions[-2, 2]
    db.update(row.id, height=h, ea=ea)



"""Module to read and write atoms in PDB file format.

See::

    http://www.wwpdb.org/documentation/file-format

Note: The PDB format saves cell lengths and angles; hence the absolute
orientation is lost when saving.  Saving and loading a file will
conserve the scaled positions, not the absolute ones.
"""

import warnings

import numpy as np

from ase.atoms import Atoms
from ase.parallel import paropen
from ase.geometry import cellpar_to_cell
from ase.utils import basestring
from ase.io.espresso import label_to_symbol


def read_atom_line(line_full):
    """
    Read atom line from pdb format
    HETATM    1  H14 ORTE    0       6.301   0.693   1.919  1.00  0.00           H
    """

    line = line_full.rstrip('\n')
    type_atm = line[0:6]
    if type_atm == "ATOM  " or type_atm == "HETATM":

        name = line[12:16].strip()

        altloc = line[16]
        resname = line[17:21]
        # chainid = line[21]        # Not used 

        resseq = int(line[22:26].split()[0])  # sequence identifier
        # icode = line[26]          # insertion code, not used

        # atomic coordinates
        try:
            coord = np.array([float(line[30:38]),
                              float(line[38:46]),
                              float(line[46:54])], dtype=np.float64)
        except ValueError:
            raise ValueError("Invalid or missing coordinate(s)")

        # occupancy & B factor
        try:
            occupancy = float(line[54:60])
        except ValueError:
            occupancy = None  # Rather than arbitrary zero or one

        if occupancy is not None and occupancy < 0:
            warnings.warn("Negative occupancy in one or more atoms")

        try:
            bfactor = float(line[60:66])
        except ValueError:
            bfactor = 0.0  # The PDB use a default of zero if the data is missing

        # segid = line[72:76] # not used
        symbol = line[76:78].strip().upper()

    else:
        raise ValueError("Only ATOM and HETATM supported")

    return symbol, name, altloc, resname, coord, occupancy, bfactor, resseq

def read_proteindatabank(fileobj, index=-1, read_arrays=True):
    """Read PDB files."""

    if isinstance(fileobj, basestring):
        fileobj = open(fileobj)

    images = []
    orig = np.identity(3)
    trans = np.zeros(3)
    occ = []
    bfactor = []
    residuenames = []
    residuenumbers = []
    atomtypes = []

    symbols = []
    positions = []
    cell = None
    pbc = None

    def build_atoms():
        atoms = Atoms(symbols=symbols,
                      cell=cell, pbc=pbc,
                      positions=positions)

        if not read_arrays:
            return atoms

        info = {'occupancy': occ,
                'bfactor': bfactor,
                'residuenames': residuenames,
                'atomtypes': atomtypes,
                'residuenumbers': residuenumbers}
        for name, array in info.items():
            if len(array) == 0:
                pass
            elif len(array) != len(atoms):
                warnings.warn('Length of {} array, {}, '
                              'different from number of atoms {}'.
                              format(name, len(array), len(atoms)))
            else:
                atoms.set_array(name, np.array(array))
        return atoms

    for line in fileobj.readlines():
        if line.startswith('CRYST1'):
            cellpar = [float(line[6:15]),  # a
                       float(line[15:24]),  # b
                       float(line[24:33]),  # c
                       float(line[33:40]),  # alpha
                       float(line[40:47]),  # beta
                       float(line[47:54])]  # gamma
            cell = cellpar_to_cell(cellpar)
            pbc = True
        for c in range(3):
            if line.startswith('ORIGX' + '123'[c]):
                orig[c] = [float(line[10:20]),
                           float(line[20:30]),
                           float(line[30:40])]
                trans[c] = float(line[45:55])

        if line.startswith('ATOM') or line.startswith('HETATM'):
            # Atom name is arbitrary and does not necessarily
            # contain the element symbol.  The specification
            # requires the element symbol to be in columns 77+78.
            # Fall back to Atom name for files that do not follow
            # the spec, e.g. packmol.

            # line_info = symbol, name, altloc, resname, coord, occupancy,
            #             bfactor, resseq
            line_info = read_atom_line(line)

            try:
                symbol = label_to_symbol(line_info[0])
            except (KeyError, IndexError):
                symbol = label_to_symbol(line_info[1])

            position = np.dot(orig, line_info[4]) + trans
            atomtypes.append(line_info[1])
            residuenames.append(line_info[3])
            if line_info[5] is not None:
                occ.append(line_info[5])
            bfactor.append(line_info[6])
            residuenumbers.append(line_info[7])

            symbols.append(symbol)
            positions.append(position)

        if line.startswith('END'):
            # End of configuration reached
            # According to the latest PDB file format (v3.30),
            # this line should start with 'ENDMDL' (not 'END'),
            # but in this way PDB trajectories from e.g. CP2K
            # are supported (also VMD supports this format).
            atoms = build_atoms()
            images.append(atoms)
            occ = []
            bfactor = []
            residuenames = []
            atomtypes = []
            symbols = []
            positions = []
            cell = None
            pbc = None

    if len(images) == 0:
        atoms = build_atoms()
        images.append(atoms)
    return images[index]


def write_proteindatabank(fileobj, images, write_arrays=True):
    """Write images to PDB-file."""
    if isinstance(fileobj, basestring):
        fileobj = paropen(fileobj, 'w')

    if hasattr(images, 'get_positions'):
        images = [images]


    rotation = None
    if images[0].get_pbc().any():
        from ase.geometry import cell_to_cellpar, cellpar_to_cell

        currentcell = images[0].get_cell()
        cellpar = cell_to_cellpar(currentcell)
        exportedcell = cellpar_to_cell(cellpar)
        rotation = np.linalg.solve(currentcell, exportedcell)
        # ignoring Z-value, using P1 since we have all atoms defined explicitly
        format = 'CRYST1%9.3f%9.3f%9.3f%7.2f%7.2f%7.2f P 1\n'
        fileobj.write(format % (cellpar[0], cellpar[1], cellpar[2],
                                cellpar[3], cellpar[4], cellpar[5]))

    #     1234567 123 6789012345678901   89   67   456789012345678901234567 890
    format = ('ATOM  %5d %4s MOL     1    %8.3f%8.3f%8.3f%6.2f%6.2f'
              '          %2s  \n')

    # RasMol complains if the atom index exceeds 100000. There might
    # be a limit of 5 digit numbers in this field.
    MAXNUM = 100000

    symbols = images[0].get_chemical_symbols()
    natoms = len(symbols)

    for n, atoms in enumerate(images):
        fileobj.write('MODEL     ' + str(n + 1) + '\n')
        p = atoms.get_positions()
        occupancy = np.ones(len(atoms))
        bfactor = np.zeros(len(atoms))
        if write_arrays:
            if 'occupancy' in atoms.arrays:
                occupancy = atoms.get_array('occupancy')
            if 'bfactor' in atoms.arrays:
                bfactor = atoms.get_array('bfactor')
        if rotation is not None:
            p = p.dot(rotation)
        for a in range(natoms):
            x, y, z = p[a]
            occ = occupancy[a]
            bf = bfactor[a]
            fileobj.write(format % (a % MAXNUM, symbols[a],
                                    x, y, z, occ, bf, symbols[a].upper()))
        fileobj.write('ENDMDL\n')



import sys
import numpy as np
from ase import Atoms
from ase.formula import Formula

assert Atoms('MoS2').get_chemical_formula() == 'MoS2'
assert Atoms('SnO2').get_chemical_formula(mode='metal') == 'SnO2'
if sys.version_info >= (3, 6):
    assert Formula('A3B2C2D').format('abc') == 'DB2C2A3'

for sym in ['', 'Pu', 'Pu2', 'U2Pu2', 'U2((Pu2)2H)']:
    for mode in ['all', 'reduce', 'hill', 'metal']:
        for empirical in [False, True]:
            if empirical and mode in ['all', 'reduce']:
                continue
            atoms = Atoms(sym)
            formula = atoms.get_chemical_formula(mode=mode,
                                                 empirical=empirical)
            atoms2 = Atoms(formula)
            print(repr(sym), '->', repr(formula))
            n1 = np.sort(atoms.numbers)
            n2 = np.sort(atoms2.numbers)
            if empirical and len(atoms) > 0:
                reduction = len(n1) // len(n2)
                n2 = np.repeat(n2, reduction)
            assert (n1 == n2).all()

for x in ['H2O', '10H2O', '2(CuO2(H2O)2)10', 'Cu20+H2', 'H' * 15,
          'AuBC2', '']:
    f = Formula(x)
    y = str(f)
    assert y == x
    print(f.count(), '{:latex}'.format(f))
    a, b = divmod(f, 'H2O')
    assert a * Formula('H2O') + b == f
    assert f != 117



# flake8: noqa
# Generated: 2011-12-19
# doi:10.1063/1.2162161, experimental geometries if available, dimers only
from numpy import array, nan
data = {
    'H': {'charges': None,
          'database': 'TMFP06D',
          'magmoms': [1.0],
          'name': 'H',
          'positions': array([[ 0.,  0.,  0.]]),
          'symbols': 'H'},
    'N': {'charges': None,
          'database': 'TMFP06D',
          'magmoms': [3.0],
          'name': 'N',
          'positions': array([[ 0.,  0.,  0.]]),
          'symbols': 'N'},
    'O': {'charges': None,
          'database': 'TMFP06D',
          'magmoms': [2.0],
          'name': 'O',
          'positions': array([[ 0.,  0.,  0.]]),
          'symbols': 'O'},
    'F': {'charges': None,
          'database': 'TMFP06D',
          'magmoms': [1.0],
          'name': 'F',
          'positions': array([[ 0.,  0.,  0.]]),
          'symbols': 'F'},
    #
    'K': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.0],
           'name': 'K',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'K'},
    'Ca': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0],
           'name': 'Ca',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Ca'},
    'Sc': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.0],
           'name': 'Sc',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Sc'},
    'Ti': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.0],
           'name': 'Ti',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Ti'},
    'V': {'charges': None,
          'database': 'TMFP06D',
          'magmoms': [3.0],
          'name': 'V',
          'positions': array([[ 0.,  0.,  0.]]),
          'symbols': 'V'},
    'Cr': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [6.0],
           'name': 'Cr',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Cr'},
    'Mn': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [5.0],
           'name': 'Mn',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Mn'},
    'Fe': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [4.0],
           'name': 'Fe',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Fe'},
    'Co': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [3.0],
           'name': 'Co',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Co'},
    'Ni': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.0],
           'name': 'Ni',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Ni'},
    'Cu': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.0],
           'name': 'Cu',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Cu'},
    'Zn': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0],
           'name': 'Zn',
           'positions': array([[ 0.,  0.,  0.]]),
           'symbols': 'Zn'},
    #
    'K2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'K2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  3.905],
                              ]),
           'dissociation energy': 12.0,
           'harmonic frequency': 92,
           'symbols': 'KK'},
    'Ca2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'Ca2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  4.277],
                              ]),
           'dissociation energy': 3.1,
           'harmonic frequency': 65,
           'symbols': 'CaCa'},
    'Sc2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.0, 2.0],
           'name': 'Sc2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.74], # from 10.1063/1.480546 (MRSDQCI)
                              ]),
           'dissociation energy': 38.4,
           'harmonic frequency': 239,
           'symbols': 'ScSc'},
    'Ti2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.0, 1.0],
           'name': 'Ti2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.943],
                              ]),
           'dissociation energy': 36.1,
           'harmonic frequency': 408,
           'symbols': 'TiTi'},
    'V2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.0, 1.0],
           'name': 'V2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.77], # ref. a
                              ]),
           'dissociation energy': 64.3,
           'harmonic frequency': 538,
           'symbols': 'VV'},
    'Cr2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'Cr2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.679],
                              ]),
           'dissociation energy': 33.9,
           'harmonic frequency': 481,
           'symbols': 'CrCr'},
    'Mn2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [5.0, 5.0],
           'name': 'Mn2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  3.40],
                              ]),
           'dissociation energy': 18.8,
           'harmonic frequency': 340,
           'symbols': 'MnMn'},
    'Fe2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [3.0, 3.0],
           'name': 'Fe2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.02],
                              ]),
           'dissociation energy': 26.9,
           'harmonic frequency': 300,
           'symbols': 'FeFe'},
    # exp. geometry not given in ref.
    'Co2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.0, 2.0],
           'name': 'Co2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.98], # calculated doi:10.1063/1.1788656
                              ]),
           'dissociation energy': 39.4,
           'harmonic frequency': 297,
           'symbols': 'CoCo'},
    'Ni2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.0, 1.0],
           'name': 'Ni2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.155],
                              ]),
           'dissociation energy': 48.1,
           'harmonic frequency': 259,
           'symbols': 'NiNi'},
    'Cu2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'Cu2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.219],
                              ]),
           'dissociation energy': 46.7,
           'harmonic frequency': 266,
           'symbols': 'CuCu'},
    'Zn2': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'Zn2',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.35], # ref. j
                              ]),
           'dissociation energy': 0.80,
           'harmonic frequency': 25.9,
           'symbols': 'ZnZn'},
    #
    'KH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'KH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.243],
                              ]),
           'dissociation energy': 42.2,
           'harmonic frequency': 707,
           'symbols': 'KH'},
    'CaH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.7, 0.3],
           'name': 'CaH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.003],
                              ]),
           'dissociation energy': 41.0,
           'harmonic frequency': 1298,
           'symbols': 'CaH'},
    # exp. geometry not given in ref.
    'ScH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'ScH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.775], # doi: 10.1063/1.3489110
                              ]),
           'dissociation energy': 47.5,
           'harmonic frequency': nan,
           'symbols': 'ScH'},
    'TiH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.9, 0.1],
           'name': 'TiH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.779],
                              ]),
           'dissociation energy': 50.0,
           'harmonic frequency': 1405,
           'symbols': 'TiH'},
    'CrH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [4.9, 0.1],
           'name': 'CrH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.656],
                              ]),
           'dissociation energy': 46.8,
           'harmonic frequency': 1581,
           'symbols': 'CrH'},
    'MnH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [5.9, 0.1],
           'name': 'MnH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.731],
                              ]),
           'dissociation energy': 31.1,
           'harmonic frequency': 1548,
           'symbols': 'MnH'},
    'FeH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.9, 0.1],
           'name': 'FeH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.589],
                              ]),
           'dissociation energy': 39.2,
           'harmonic frequency': 1827,
           'symbols': 'FeH'},
    'CoH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.9, 0.1],
           'name': 'CoH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.52],
                              ]),
           'dissociation energy': 48.4,
           'harmonic frequency': 1925,
           'symbols': 'CoH'},
    'NiH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.9, 0.1],
           'name': 'NiH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.475],
                              ]),
           'dissociation energy': 61.3,
           'harmonic frequency': 1927,
           'symbols': 'NiH'},
    'CuH': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'CuH',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.463],
                              ]),
           'dissociation energy': 63.4,
           'harmonic frequency': 1941,
           'symbols': 'CuH'},
    #
    'ScN': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'ScN',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.687],
                              ]),
           'dissociation energy': 113,
           'harmonic frequency': 795,
           'symbols': 'ScN'},
    'TiN': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.9, 0.1],
           'name': 'TiN',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.57],
                              ]),
           'dissociation energy': 124,
           'harmonic frequency': 1033,
           'symbols': 'TiN'},
    'VN': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.9, 0.1],
           'name': 'VN',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.57],
                              ]),
           'dissociation energy': 117,
           'harmonic frequency': 1033,
           'symbols': 'VN'},
    'CrN': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.9, 0.1],
           'name': 'CrN',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.563],
                              ]),
           'dissociation energy': 97,
           'harmonic frequency': 1050,
           'symbols': 'CrN'},
    #
    'CaO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'CaO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.822],
                              ]),
           'dissociation energy': 110.8,
           'harmonic frequency': 732,
            'symbols': 'CaO'},
    'ScO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.9, 0.1],
           'name': 'ScO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.668],
                              ]),
           'dissociation energy': 163.0,
           'harmonic frequency': 965,
            'symbols': 'ScO'},
    'TiO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.9, 0.1],
           'name': 'TiO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.620],
                              ]),
           'dissociation energy': 161.0,
           'harmonic frequency': 1009,
            'symbols': 'TiO'},
    'VO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.9, 0.1],
           'name': 'VO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.589],
                              ]),
           'dissociation energy': 150.0,
           'harmonic frequency': 1011,
            'symbols': 'VO'},
    'CrO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [3.9, 0.1],
           'name': 'CrO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.615],
                              ]),
           'dissociation energy': 103.0,
           'harmonic frequency': 898,
            'symbols': 'CrO'},
    'MnO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [4.9, 0.1],
           'name': 'MnO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.646],
                              ]),
           'dissociation energy': 89.5,
           'harmonic frequency': 840,
            'symbols': 'MnO'},
    'FeO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [3.9, 0.1],
           'name': 'FeO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.616],
                              ]),
           'dissociation energy': 97.4,
           'harmonic frequency': 880,
            'symbols': 'FeO'},
    'CoO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.5, 0.5],
           'name': 'CoO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.629],
                              ]),
           'dissociation energy': 92.1,
           'harmonic frequency': 853,
            'symbols': 'CoO'},
    'NiO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [1.0, 1.0],
           'name': 'NiO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.627],
                              ]),
           'dissociation energy': 90.4,
           'harmonic frequency': 838,
            'symbols': 'NiO'},
    'CuO': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.5, 0.5],
           'name': 'CuO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.724],
                              ]),
           'dissociation energy': 66.6,
           'harmonic frequency': 640,
            'symbols': 'CuO'},
    # ZnO not in ref. Taken for completeness from doi: 10.1063/1.3489110
    'ZnO': {'charges': None,
           'database': 'TMKM11',
           'magmoms': [0.0, 0.0],
           'name': 'ZnO',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.724],
                              ]),
           'dissociation energy': nan,
           'harmonic frequency': nan,
           'symbols': 'ZnO'},
    #
    'KF': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'KF',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  2.171],
                              ]),
           'dissociation energy': 117.5,
           'harmonic frequency': 428,
            'symbols': 'KF'},
    'CaF': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.9, 0.1],
           'name': 'CaF',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.967],
                              ]),
           'dissociation energy': 127.2,
           'harmonic frequency': 581,
            'symbols': 'CaF'},
    'ScF': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'ScF',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.788],
                              ]),
           'dissociation energy': 143.3,
           'harmonic frequency': 736,
            'symbols': 'ScF'},
    'TiF': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [2.9, 0.1],
           'name': 'TiF',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.834],
                              ]),
           'dissociation energy': 137,
           'harmonic frequency': nan,
            'symbols': 'TiF'},
    'CrF': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [4.9, 0.1],
           'name': 'CrF',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.784],
                              ]),
           'dissociation energy': 106.3,
           'harmonic frequency': 664,
            'symbols': 'CrF'},
    'FeF': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [4.9, 0.1],
           'name': 'FeF',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.780],
                              ]),
           'dissociation energy': nan,
           'harmonic frequency': 663,
            'symbols': 'FeF'},
    'CuF': {'charges': None,
           'database': 'TMFP06D',
           'magmoms': [0.0, 0.0],
           'name': 'CuF',
           'positions': array([
                              [ 0.,  0.,  0.],
                              [ 0.,  0.,  1.745],
                              ]),
           'dissociation energy': 102.8,
           'harmonic frequency': 623,
            'symbols': 'CuF'},
    }



"""Structure data - Hexagonal Closed Packed"""

import numpy as np

from ase.cluster.data.symmetry import (get_all_symmetries,
                                       get_surface_symmetries,
                                       get_neighbor_symmetries,
                                       apply_neighbor_symmetry)


# Definition of symmetries (in hcp basis)
basesymmetries = [np.array([[1, 0, 0],  # Mirror y-axis
                            [0, 1, 0],
                            [0, 0, -1]]),
                  np.array([[0, 1, 0],  # Rotation z-axix (3-fold)
                            [-1, -1, 0],
                            [0, 0, 1]]),
                  np.array([[1, 0, 0],  # Rotation a-axis (2-fold)
                            [-1, -1, 0],
                            [0, 0, -1]]),
                  np.array([[-1, -1, 0],  # Rotation b-axis (2-fold)
                            [0, 1, 0],
                            [0, 0, -1]]),
                  np.array([[0, 1, 0],  # Rotation ab-axis (2-fold)
                            [1, 0, 0],
                            [0, 0, -1]])]

symmetries = get_all_symmetries(basesymmetries, 12)

# Definition of surfaces
surface_names = [(0, 0, 1), (0, 0, -1),  # (001)
                 (1, -1, 0), (-1, 1, 0),
                 (2, 1, 0), (-2, -1, 0),
                 (1, 2, 0), (-1, -2, 0),
                 (5, -5, 3), (5, -5, -3),
                 (-5, 5, 3), (-5, 5, -3),
                 (5, 10, 3), (5, 10, -3),
                 (10, 5, 3), (10, 5, -3),
                 (-5, -10, 3), (-5, -10, -3),
                 (-10, -5, 3), (-10, -5, -3)]

surface_numbers = {}
for i, s in enumerate(surface_names):
    surface_numbers[s] = i

surface_count = len(surface_names)

surface_mapping = {0: 1, 1: 0, 2: 3, 3: 2, 4: 5, 5: 4, 6: 7, 7: 6}

surface_data = ([{'d': 0.5}] * 2 +
                [{'d': 0.5}] * 6 +
                [{'d': 1 / 13}] * 12)

surface_symmetries = get_surface_symmetries(symmetries, surface_names,
                                            surface_numbers)

# Definition of neighbor environment
neighbor_names = [(1, 0, 0), (-1, 0, 0),
                  (0, 1, 0), (0, -1, 0),
                  (1, 1, 0), (-1, -1, 0),
                  (1 / 3, 2 / 3, 1 / 2), (1 / 3, -1 / 3, 1 / 2),
                  (-2 / 3, -1 / 3, 1 / 2),
                  (1 / 3, 2 / 3, -1 / 2), (1 / 3, -1 / 3, -1 / 2),
                  (-2 / 3, -1 / 3, -1 / 2),
                  (2 / 3, 1 / 3, 1 / 2), (-1 / 3, -2 / 3, 1 / 2),
                  (-1 / 3, 1 / 3, 1 / 2),
                  (2 / 3, 1 / 3, -1 / 2), (-1 / 3, -2 / 3, -1 / 2),
                  (-1 / 3, 1 / 3, -1 / 2)]

neighbor_numbers = {}
for i, n in enumerate(neighbor_names):
    neighbor_numbers[n] = i

neighbor_positions = np.array(neighbor_names, dtype=float)

neighbor_cutoff = 1.2
neighbor_count = 16

neighbor_mapping = {0: 1, 1: 0, 2: 3, 3: 2, 4: 5, 5: 4,
                    6: 16, 7: 17, 8: 15, 9: 13, 10: 14, 11: 12,
                    12: 11, 13: 9, 14: 10, 15: 8, 16: 6, 17: 7}

neighbor_symmetries = get_neighbor_symmetries(symmetries,
                                              neighbor_positions,
                                              neighbor_numbers)

# Definition of the atom types that is used based on the neighborlist
basetype_names = []

basetype_data = []

type_count = len(basetype_names)

type_names = []
type_data = []
for i, n in enumerate(basetype_names):
    type_names.append(n)
    type_data.append(basetype_data[i])

    for sym in neighbor_symmetries:
        new_type = apply_neighbor_symmetry(n, sym)

        if new_type not in type_names:
            type_names.append(new_type)
            type_data.append(basetype_data[i])

type_numbers = {}
for i, n in enumerate(type_names):
    type_numbers[n] = i

# Collect all data
data = {'symmetries': symmetries,
        'surface_names': surface_names,
        'surface_numbers': surface_numbers,
        'surface_data': surface_data,
        'surface_count': surface_count,
        'surface_mapping': surface_mapping,
        'surface_symmetries': surface_symmetries,
        'neighbor_positions': neighbor_positions,
        'neighbor_numbers': neighbor_numbers,
        'neighbor_count': neighbor_count,
        'neighbor_cutoff': neighbor_cutoff,
        'neighbor_mapping': neighbor_mapping,
        'neighbor_symmetries': neighbor_symmetries,
        'type_names': type_names,
        'type_numbers': type_numbers,
        'type_data': type_data,
        'type_count': type_count}



from ase.utils import basestring
import re
import warnings
from math import sin, cos

import numpy as np

from ase.utils import jsonable
from ase.cell import Cell
from ase.geometry import cell_to_cellpar, crystal_structure_from_cell


def monkhorst_pack(size):
    """Construct a uniform sampling of k-space of given size."""
    if np.less_equal(size, 0).any():
        raise ValueError('Illegal size: %s' % list(size))
    kpts = np.indices(size).transpose((1, 2, 3, 0)).reshape((-1, 3))
    return (kpts + 0.5) / size - 0.5


def get_monkhorst_pack_size_and_offset(kpts):
    """Find Monkhorst-Pack size and offset.

    Returns (size, offset), where::

        kpts = monkhorst_pack(size) + offset.

    The set of k-points must not have been symmetry reduced."""

    if len(kpts) == 1:
        return np.ones(3, int), np.array(kpts[0], dtype=float)

    size = np.zeros(3, int)
    for c in range(3):
        # Determine increment between k-points along current axis
        delta = max(np.diff(np.sort(kpts[:, c])))

        # Determine number of k-points as inverse of distance between kpoints
        if delta > 1e-8:
            size[c] = int(round(1.0 / delta))
        else:
            size[c] = 1

    if size.prod() == len(kpts):
        kpts0 = monkhorst_pack(size)
        offsets = kpts - kpts0

        # All offsets must be identical:
        if (offsets.ptp(axis=0) < 1e-9).all():
            return size, offsets[0].copy()

    raise ValueError('Not an ASE-style Monkhorst-Pack grid!')


def get_monkhorst_shape(kpts):
    warnings.warn('Use get_monkhorst_pack_size_and_offset()[0] instead.')
    return get_monkhorst_pack_size_and_offset(kpts)[0]


def kpoint_convert(cell_cv, skpts_kc=None, ckpts_kv=None):
    """Convert k-points between scaled and cartesian coordinates.

    Given the atomic unit cell, and either the scaled or cartesian k-point
    coordinates, the other is determined.

    The k-point arrays can be either a single point, or a list of points,
    i.e. the dimension k can be empty or multidimensional.
    """
    if ckpts_kv is None:
        icell_cv = 2 * np.pi * np.linalg.pinv(cell_cv).T
        return np.dot(skpts_kc, icell_cv)
    elif skpts_kc is None:
        return np.dot(ckpts_kv, cell_cv.T) / (2 * np.pi)
    else:
        raise KeyError('Either scaled or cartesian coordinates must be given.')


def parse_path_string(s):
    """Parse compact string representation of BZ path.

    A path string can have several non-connected sections separated by
    commas. The return value is a list of sections where each section is a
    list of labels.

    Examples:

    >>> parse_path_string('GX')
    [['G', 'X']]
    >>> parse_path_string('GX,M1A')
    [['G', 'X'], ['M1', 'A']]
    """
    paths = []
    for path in s.split(','):
        names = [name
                 for name in re.split(r'([A-Z][a-z0-9]*)', path)
                 if name]
        paths.append(names)
    return paths


def resolve_kpt_path_string(path, special_points):
    paths = parse_path_string(path)
    coords = [np.array([special_points[sym] for sym in subpath]).reshape(-1, 3)
              for subpath in paths]
    return paths, coords


def resolve_custom_points(pathspec, special_points, eps):
    """Resolve a path specification into a string.

    The path specification is a list path segments, each segment being a kpoint
    label or kpoint coordinate, or a single such segment.

    Return a string representing the same path.  Generic kpoint labels
    are generated dynamically as necessary, updating the special_point
    dictionary if necessary.  The tolerance eps is used to see whether
    coordinates are close enough to a special point to deserve being
    labelled as such."""
    # This should really run on Cartesian coordinates but we'll probably
    # be lazy and call it on scaled ones.

    if len(pathspec) == 0:
        return ''

    nested_format = True
    for element in pathspec:
        if len(element) == 3 and np.isscalar(element[0]):
            nested_format = False
            break

    if not nested_format:
        pathspec = [pathspec]  # Now format is nested.

    def name_generator():
        counter = 0
        while True:
            name = 'Kpt{}'.format(counter)
            yield name
            counter += 1
    custom_names = name_generator()

    labelseq = []
    for segment in pathspec:
        for kpt in segment:
            if isinstance(kpt, str):
                if kpt not in special_points:
                    raise KeyError('No kpoint "{}" among "{}"'.format(kpt),
                                   ''.join(special_points))
                labelseq.append(kpt)
                continue

            kpt = np.asarray(kpt, float)
            for key, val in special_points.items():
                if np.abs(kpt - val).max() < eps:
                    labelseq.append(key)
                    break  # Already present
            else:
                # New special point - search for name we haven't used yet:
                name = next(custom_names)
                while name in special_points:
                    name = next(custom_names)
                special_points[name] = kpt
                labelseq.append(name)
        labelseq.append(',')

    last = labelseq.pop()
    assert last == ','
    return ''.join(labelseq)


@jsonable('bandpath')
class BandPath:
    """Represents a Brillouin zone path or bandpath.

    A band path has a unit cell, a path specification, special points,
    and interpolated k-points.  Band paths are typically created
    indirectly using the :class:`~ase.geometry.Cell` or
    :class:`~ase.lattice.BravaisLattice` classes:

    >>> from ase.lattice import CUB
    >>> path = CUB(3).bandpath()
    >>> path
    BandPath(path='GXMGRX,MR', cell=[3x3], special_points={GMRX}, kpts=[40x3])

    Band paths support JSON I/O:

    >>> from ase.io.jsonio import read_json
    >>> path.write('mybandpath.json')
    >>> read_json('mybandpath.json')
    BandPath(path='GXMGRX,MR', cell=[3x3], special_points={GMRX}, kpts=[40x3])

    """
    def __init__(self, cell, kpts=None,
                 special_points=None, path=None):
        if kpts is None:
            kpts = np.empty((0, 3))

        if special_points is None:
            special_points = {}
        else:
            special_points = dict(special_points)

        if path is None:
            path = ''

        self.cell = cell = Cell.new(cell)
        assert cell.shape == (3, 3)
        kpts = np.asarray(kpts)
        assert kpts.ndim == 2 and kpts.shape[1] == 3
        self.icell = self.cell.reciprocal()
        self.kpts = kpts
        self.special_points = special_points
        assert isinstance(path, str)
        self.path = path

    def transform(self, op):
        """Apply 3x3 matrix to this BandPath and return new BandPath.

        This is useful for converting the band path to another cell.
        The operation will typically be a permutation/flipping
        established by a function such as Niggli reduction."""
        # XXX acceptable operations are probably only those
        # who come from Niggli reductions (permutations etc.)
        #
        # We should insert a check.
        # I wonder which operations are valid?  They won't be valid
        # if they change lengths, volume etc.
        special_points = {}
        for name, value in self.special_points.items():
            special_points[name] = value @ op

        return BandPath(op.T @ self.cell, kpts=self.kpts @ op,
                        special_points=special_points,
                        path=self.path)

    def todict(self):
        return {'kpts': self.kpts,
                'special_points': self.special_points,
                'labelseq': self.path,
                'cell': self.cell}

    def interpolate(self, path=None, npoints=None, special_points=None,
                    density=None):
        """Create new bandpath, (re-)interpolating kpoints from this one."""
        if path is None:
            path = self.path

        special_points = {} if special_points is None else dict(special_points)
        special_points.update(self.special_points)
        pathnames, pathcoords = resolve_kpt_path_string(path, special_points)
        kpts, x, X = paths2kpts(pathcoords, self.cell, npoints, density)
        return BandPath(self.cell, kpts, path=path,
                        special_points=special_points)

    def _scale(self, coords):
        return np.dot(coords, self.icell)

    def __repr__(self):
        return ('{}(path={}, cell=[3x3], special_points={{{}}}, kpts=[{}x3])'
                .format(self.__class__.__name__,
                        repr(self.path),
                        ''.join(sorted(self.special_points)),
                        len(self.kpts)))

    def cartesian_kpts(self):
        """Get Cartesian kpoints from this bandpath."""
        return self._scale(self.kpts)

    def __iter__(self):
        """XXX Compatibility hack for bandpath() function.

        bandpath() now returns a BandPath object, which is a Good
        Thing.  However it used to return a tuple of (kpts, x_axis,
        special_x_coords), and people would use tuple unpacking for
        those.

        This function makes tuple unpacking work in the same way.
        It will be removed in the future.

        """
        import warnings
        warnings.warn('Please do not use (kpts, x, X) = bandpath(...).  '
                      'Use path = bandpath(...) and then kpts = path.kpts and '
                      '(x, X, labels) = path.get_linear_kpoint_axis().')
        yield self.kpts

        x, xspecial, _ = labels_from_kpts(self.kpts, self.cell,
                                          special_points=self.special_points)
        yield x
        yield xspecial

    def __getitem__(self, index):
        # Temp compatibility stuff, see __iter__
        return tuple(self)[index]

    def get_linear_kpoint_axis(self, eps=1e-5):
        """Define x axis suitable for plotting a band structure.

        See :func:`ase.dft.kpoints.labels_from_kpts`."""
        return labels_from_kpts(self.kpts, self.cell, eps=eps,
                                special_points=self.special_points)

    def plot(self, dimension=3, **plotkwargs):
        """Visualize this bandpath.

        Plots the irreducible Brillouin zone and this bandpath."""
        import ase.dft.bz as bz

        special_points = self.special_points
        labelseq, coords = resolve_kpt_path_string(self.path,
                                                   special_points)

        paths = []
        points_already_plotted = set()
        for subpath_labels, subpath_coords in zip(labelseq, coords):
            subpath_coords = np.array(subpath_coords)
            points_already_plotted.update(subpath_labels)
            paths.append((subpath_labels, self._scale(subpath_coords)))

        # Add each special point as a single-point subpath if they were
        # not plotted already:
        for label, point in special_points.items():
            if label not in points_already_plotted:
                paths.append(([label], [self._scale(point)]))

        kw = {'vectors': True}
        kw.update(plotkwargs)
        return bz.bz_plot(self.cell, paths=paths,
                          points=self.cartesian_kpts(),
                          pointstyle={'marker': '.'},
                          **kw)

    def free_electron_band_structure(self, **kwargs):
        """Return band structure of free electrons for this bandpath.

        This is for mostly testing."""
        from ase import Atoms
        from ase.calculators.test import FreeElectrons
        from ase.dft.band_structure import calculate_band_structure
        atoms = Atoms(cell=self.cell, pbc=True)
        atoms.calc = FreeElectrons(**kwargs)
        bs = calculate_band_structure(atoms, path=self)
        return bs


def bandpath(path, cell, npoints=None, density=None, special_points=None,
             eps=2e-4):
    """Make a list of kpoints defining the path between the given points.

    path: list or str
        Can be:

        * a string that parse_path_string() understands: 'GXL'
        * a list of BZ points: [(0, 0, 0), (0.5, 0, 0)]
        * or several lists of BZ points if the the path is not continuous.
    cell: 3x3
        Unit cell of the atoms.
    npoints: int
        Length of the output kpts list. If too small, at least the beginning
        and ending point of each path segment will be used. If None (default),
        it will be calculated using the supplied density or a default one.
    density: float
        k-points per A⁻¹ on the output kpts list. If npoints is None,
        the number of k-points in the output list will be:
        npoints = density * path total length (in Angstroms).
        If density is None (default), use 5 k-points per A⁻¹.
        If the calculated npoints value is less than 50, a mimimum value of 50
        will be used.
    special_points: dict or None
        Dictionary mapping names to special points.  If None, the special
        points will be derived from the cell.
    eps: float
        Precision used to identify Bravais lattice, deducing special points.

    You may define npoints or density but not both.

    Return a :class:`~ase.dft.kpoints.BandPath` object."""

    cell = Cell.ascell(cell)
    return cell.bandpath(path, npoints=npoints, density=density,
                         special_points=special_points, eps=eps)

    # XXX old code for bandpath() function, should be removed once we
    # weed out any trouble
    if isinstance(path, basestring):
        # XXX we need to update this so we use the new and more complete
        # cell classification stuff
        lattice = None
        if special_points is None:
            cell = Cell.ascell(cell)
            cellinfo = get_cellinfo(cell)
            special_points = cellinfo.special_points
            lattice = cellinfo.lattice
        paths = []
        for names in parse_path_string(path):
            for name in names:
                if name not in special_points:
                    msg = ('K-point label {} not included in {} special '
                           'points.  Valid labels are: {}'
                           .format(name, lattice or 'custom dictionary of',
                                   ', '.join(sorted(special_points))))
                    raise ValueError(msg)
            paths.append([special_points[name] for name in names])
    elif np.array(path[0]).ndim == 1:
        paths = [path]
    else:
        paths = path

    kpts, x, X = paths2kpts(paths, cell, npoints, density)
    return BandPath(cell, kpts=kpts, special_points=special_points)


DEFAULT_KPTS_DENSITY = 5    # points per 1/Angstrom


def paths2kpts(paths, cell, npoints=None, density=None):
    if not(npoints is None or density is None):
        raise ValueError('You may define npoints or density, but not both.')
    points = np.concatenate(paths)
    dists = points[1:] - points[:-1]
    lengths = [np.linalg.norm(d) for d in kpoint_convert(cell, skpts_kc=dists)]

    i = 0
    for path in paths[:-1]:
        i += len(path)
        lengths[i - 1] = 0

    length = sum(lengths)

    if npoints is None:
        if density is None:
            density = DEFAULT_KPTS_DENSITY
        # Set npoints using the length of the path
        npoints = int(round(length * density))

    kpts = []
    x0 = 0
    x = []
    X = [0]
    for P, d, L in zip(points[:-1], dists, lengths):
        diff = length - x0
        if abs(diff) < 1e-6:
            n = 0
        else:
            n = max(2, int(round(L * (npoints - len(x)) / diff)))

        for t in np.linspace(0, 1, n)[:-1]:
            kpts.append(P + t * d)
            x.append(x0 + t * L)
        x0 += L
        X.append(x0)
    if len(points):
        kpts.append(points[-1])
        x.append(x0)

    if len(kpts) == 0:
        kpts = np.empty((0, 3))

    return np.array(kpts), np.array(x), np.array(X)


get_bandpath = bandpath  # old name


def find_bandpath_kinks(cell, kpts, eps=1e-5):
    """Find indices of those kpoints that are not interiour to a line segment."""
    diffs = kpts[1:] - kpts[:-1]
    kinks = abs(diffs[1:] - diffs[:-1]).sum(1) > eps
    N = len(kpts)
    indices = []
    if N > 0:
        indices.append(0)
        indices.extend(np.arange(1, N - 1)[kinks])
        indices.append(N - 1)
    return indices


def labels_from_kpts(kpts, cell, eps=1e-5, special_points=None):
    """Get an x-axis to be used when plotting a band structure.

    The first of the returned lists can be used as a x-axis when plotting
    the band structure. The second list can be used as xticks, and the third
    as xticklabels.

    Parameters:

    kpts: list
        List of scaled k-points.

    cell: list
        Unit cell of the atomic structure.

    Returns:

    Three arrays; the first is a list of cumulative distances between k-points,
    the second is x coordinates of the special points,
    the third is the special points as strings.
    """


    if special_points is None:
        special_points = get_special_points(cell)
    points = np.asarray(kpts)
    indices = find_bandpath_kinks(cell, kpts, eps=1e-5)

    labels = []
    for kpt in points[indices]:
        for label, k in special_points.items():
            if abs(kpt - k).sum() < eps:
                break
        else:
            # No exact match.  Try modulus 1:
            for label, k in special_points.items():
                if abs((kpt - k) % 1).sum() < eps:
                    break
            else:
                label = '?'
        labels.append(label)

    jump = False  # marks a discontinuity in the path
    xcoords = [0]
    for i1, i2 in zip(indices[:-1], indices[1:]):
        if not jump and i1 + 1 == i2:
            length = 0
            jump = True  # we don't want two jumps in a row
        else:
            diff = points[i2] - points[i1]
            length = np.linalg.norm(kpoint_convert(cell, skpts_kc=diff))
            jump = False
        xcoords.extend(np.linspace(0, length, i2 - i1 + 1)[1:] + xcoords[-1])

    xcoords = np.array(xcoords)
    return xcoords, xcoords[indices], labels


special_paths = {
    'cubic': 'GXMGRX,MR',
    'fcc': 'GXWKGLUWLK,UX',
    'bcc': 'GHNGPH,PN',
    'tetragonal': 'GXMGZRAZXR,MA',
    'orthorhombic': 'GXSYGZURTZ,YT,UX,SR',
    'hexagonal': 'GMKGALHA,LM,KH',
    'monoclinic': 'GYHCEM1AXH1,MDZ,YD',
    'rhombohedral type 1': 'GLB1,BZGX,QFP1Z,LP',
    'rhombohedral type 2': 'GPZQGFP1Q1LZ'}


class CellInfo:
    def __init__(self, rcell, lattice, special_points):
        self.rcell = rcell
        self.lattice = lattice
        self.special_points = special_points

def get_cellinfo(cell, lattice=None, eps=2e-4):
    from ase.build.tools import niggli_reduce_cell
    warnings.warn(
        "This function is deprecated, use ase.lattice or get_bandpath",
        np.VisibleDeprecationWarning
    )
    rcell, M = niggli_reduce_cell(cell)
    latt = crystal_structure_from_cell(rcell, niggli_reduce=False)
    if lattice:
        assert latt == lattice.lower(), latt

    if latt == 'monoclinic':
        # Transform From Niggli to Setyawana-Curtarolo cell:
        a, b, c, alpha, beta, gamma = cell_to_cellpar(rcell, radians=True)
        if abs(beta - np.pi / 2) > eps:
            T = np.array([[0, 1, 0],
                          [-1, 0, 0],
                          [0, 0, 1]])
            scell = np.dot(T, rcell)
        elif abs(gamma - np.pi / 2) > eps:
            T = np.array([[0, 0, 1],
                          [1, 0, 0],
                          [0, -1, 0]])
        else:
            raise ValueError('You are using a badly oriented ' +
                             'monoclinic unit cell. Please choose one with ' +
                             'either beta or gamma != pi/2')

        scell = np.dot(np.dot(T, rcell), T.T)
        a, b, c, alpha, beta, gamma = cell_to_cellpar(scell, radians=True)

        assert alpha < np.pi / 2, 'Your monoclinic angle has to be < pi / 2'

        M = np.dot(M, T.T)
        eta = (1 - b * cos(alpha) / c) / (2 * sin(alpha)**2)
        nu = 1 / 2 - eta * c * cos(alpha) / b
        points = {'G': [0, 0, 0],
                  'A': [1 / 2, 1 / 2, 0],
                  'C': [0, 1 / 2, 1 / 2],
                  'D': [1 / 2, 0, 1 / 2],
                  'D1': [1 / 2, 0, -1 / 2],
                  'E': [1 / 2, 1 / 2, 1 / 2],
                  'H': [0, eta, 1 - nu],
                  'H1': [0, 1 - eta, nu],
                  'H2': [0, eta, -nu],
                  'M': [1 / 2, eta, 1 - nu],
                  'M1': [1 / 2, 1 - eta, nu],
                  'M2': [1 / 2, eta, -nu],
                  'X': [0, 1 / 2, 0],
                  'Y': [0, 0, 1 / 2],
                  'Y1': [0, 0, -1 / 2],
                  'Z': [1 / 2, 0, 0]}
    elif latt == 'rhombohedral type 1':
        a, b, c, alpha, beta, gamma = cell_to_cellpar(cell=cell, radians=True)
        eta = (1 + 4 * np.cos(alpha)) / (2 + 4 * np.cos(alpha))
        nu = 3 / 4 - eta / 2
        points = {'G': [0, 0, 0],
                  'B': [eta, 1 / 2, 1 - eta],
                  'B1': [1 / 2, 1 - eta, eta - 1],
                  'F': [1 / 2, 1 / 2, 0],
                  'L': [1 / 2, 0, 0],
                  'L1': [0, 0, - 1 / 2],
                  'P': [eta, nu, nu],
                  'P1': [1 - nu, 1 - nu, 1 - eta],
                  'P2': [nu, nu, eta - 1],
                  'Q': [1 - nu, nu, 0],
                  'X': [nu, 0, -nu],
                  'Z': [0.5, 0.5, 0.5]}
    else:
        points = sc_special_points[latt]

    myspecial_points = {label: np.dot(M, kpt) for label, kpt in points.items()}
    return CellInfo(rcell=rcell, lattice=latt,
                    special_points=myspecial_points)


def get_special_points(cell, lattice=None, eps=2e-4):
    """Return dict of special points.

    The definitions are from a paper by Wahyu Setyawana and Stefano
    Curtarolo::

        http://dx.doi.org/10.1016/j.commatsci.2010.05.010

    cell: 3x3 ndarray
        Unit cell.
    lattice: str
        Optionally check that the cell is one of the following: cubic, fcc,
        bcc, orthorhombic, tetragonal, hexagonal or monoclinic.
    eps: float
        Tolerance for cell-check.
    """

    if isinstance(cell, str):
        warnings.warn('Please call this function with cell as the first '
                      'argument')
        lattice, cell = cell, lattice

    cell = Cell.ascell(cell)
    # We create the bandpath because we want to transform the kpoints too,
    # from the canonical cell to the given one.
    #
    # Note that this function is missing a tolerance, epsilon.
    path = cell.bandpath(npoints=0)
    return path.special_points


def monkhorst_pack_interpolate(path, values, icell, bz2ibz,
                               size, offset=(0, 0, 0)):
    """Interpolate values from Monkhorst-Pack sampling.

    path: (nk, 3) array-like
        Desired path in units of reciprocal lattice vectors.
    values: (nibz, ...) array-like
        Values on Monkhorst-Pack grid.
    icell: (3, 3) array-like
        Reciprocal lattice vectors.
    bz2ibz: (nbz,) array-like of int
        Map from nbz points in BZ to nibz reduced points in IBZ.
    size: (3,) array-like of int
        Size of Monkhorst-Pack grid.
    offset: (3,) array-like
        Offset of Monkhorst-Pack grid.

    Returns *values* interpolated to *path*.
    """
    from scipy.interpolate import LinearNDInterpolator

    path = (np.asarray(path) + 0.5) % 1 - 0.5
    path = np.dot(path, icell)

    # Fold out values from IBZ to BZ:
    v = np.asarray(values)[bz2ibz]
    v = v.reshape(tuple(size) + v.shape[1:])

    # Create padded Monkhorst-Pack grid:
    size = np.asarray(size)
    i = np.indices(size + 2).transpose((1, 2, 3, 0)).reshape((-1, 3))
    k = (i - 0.5) / size - 0.5 + offset
    k = np.dot(k, icell)

    # Fill in boundary values:
    V = np.zeros(tuple(size + 2) + v.shape[3:])
    V[1:-1, 1:-1, 1:-1] = v
    V[0, 1:-1, 1:-1] = v[-1]
    V[-1, 1:-1, 1:-1] = v[0]
    V[:, 0, 1:-1] = V[:, -2, 1:-1]
    V[:, -1, 1:-1] = V[:, 1, 1:-1]
    V[:, :, 0] = V[:, :, -2]
    V[:, :, -1] = V[:, :, 1]

    interpolate = LinearNDInterpolator(k, V.reshape((-1,) + V.shape[3:]))
    return interpolate(path)


# ChadiCohen k point grids. The k point grids are given in units of the
# reciprocal unit cell. The variables are named after the following
# convention: cc+'<Nkpoints>'+_+'shape'. For example an 18 k point
# sq(3)xsq(3) is named 'cc18_sq3xsq3'.

cc6_1x1 = np.array([
    1, 1, 0, 1, 0, 0, 0, -1, 0, -1, -1, 0, -1, 0, 0,
    0, 1, 0]).reshape((6, 3)) / 3.0

cc12_2x3 = np.array([
    3, 4, 0, 3, 10, 0, 6, 8, 0, 3, -2, 0, 6, -4, 0,
    6, 2, 0, -3, 8, 0, -3, 2, 0, -3, -4, 0, -6, 4, 0, -6, -2, 0, -6,
    -8, 0]).reshape((12, 3)) / 18.0

cc18_sq3xsq3 = np.array([
    2, 2, 0, 4, 4, 0, 8, 2, 0, 4, -2, 0, 8, -4,
    0, 10, -2, 0, 10, -8, 0, 8, -10, 0, 2, -10, 0, 4, -8, 0, -2, -8,
    0, 2, -4, 0, -4, -4, 0, -2, -2, 0, -4, 2, 0, -2, 4, 0, -8, 4, 0,
    -4, 8, 0]).reshape((18, 3)) / 18.0

cc18_1x1 = np.array([
    2, 4, 0, 2, 10, 0, 4, 8, 0, 8, 4, 0, 8, 10, 0,
    10, 8, 0, 2, -2, 0, 4, -4, 0, 4, 2, 0, -2, 8, 0, -2, 2, 0, -2, -4,
    0, -4, 4, 0, -4, -2, 0, -4, -8, 0, -8, 2, 0, -8, -4, 0, -10, -2,
    0]).reshape((18, 3)) / 18.0

cc54_sq3xsq3 = np.array([
    4, -10, 0, 6, -10, 0, 0, -8, 0, 2, -8, 0, 6,
    -8, 0, 8, -8, 0, -4, -6, 0, -2, -6, 0, 2, -6, 0, 4, -6, 0, 8, -6,
    0, 10, -6, 0, -6, -4, 0, -2, -4, 0, 0, -4, 0, 4, -4, 0, 6, -4, 0,
    10, -4, 0, -6, -2, 0, -4, -2, 0, 0, -2, 0, 2, -2, 0, 6, -2, 0, 8,
    -2, 0, -8, 0, 0, -4, 0, 0, -2, 0, 0, 2, 0, 0, 4, 0, 0, 8, 0, 0,
    -8, 2, 0, -6, 2, 0, -2, 2, 0, 0, 2, 0, 4, 2, 0, 6, 2, 0, -10, 4,
    0, -6, 4, 0, -4, 4, 0, 0, 4, 0, 2, 4, 0, 6, 4, 0, -10, 6, 0, -8,
    6, 0, -4, 6, 0, -2, 6, 0, 2, 6, 0, 4, 6, 0, -8, 8, 0, -6, 8, 0,
    -2, 8, 0, 0, 8, 0, -6, 10, 0, -4, 10, 0]).reshape((54, 3)) / 18.0

cc54_1x1 = np.array([
    2, 2, 0, 4, 4, 0, 8, 8, 0, 6, 8, 0, 4, 6, 0, 6,
    10, 0, 4, 10, 0, 2, 6, 0, 2, 8, 0, 0, 2, 0, 0, 4, 0, 0, 8, 0, -2,
    6, 0, -2, 4, 0, -4, 6, 0, -6, 4, 0, -4, 2, 0, -6, 2, 0, -2, 0, 0,
    -4, 0, 0, -8, 0, 0, -8, -2, 0, -6, -2, 0, -10, -4, 0, -10, -6, 0,
    -6, -4, 0, -8, -6, 0, -2, -2, 0, -4, -4, 0, -8, -8, 0, 4, -2, 0,
    6, -2, 0, 6, -4, 0, 2, 0, 0, 4, 0, 0, 6, 2, 0, 6, 4, 0, 8, 6, 0,
    8, 0, 0, 8, 2, 0, 10, 4, 0, 10, 6, 0, 2, -4, 0, 2, -6, 0, 4, -6,
    0, 0, -2, 0, 0, -4, 0, -2, -6, 0, -4, -6, 0, -6, -8, 0, 0, -8, 0,
    -2, -8, 0, -4, -10, 0, -6, -10, 0]).reshape((54, 3)) / 18.0

cc162_sq3xsq3 = np.array([
    -8, 16, 0, -10, 14, 0, -7, 14, 0, -4, 14,
    0, -11, 13, 0, -8, 13, 0, -5, 13, 0, -2, 13, 0, -13, 11, 0, -10,
    11, 0, -7, 11, 0, -4, 11, 0, -1, 11, 0, 2, 11, 0, -14, 10, 0, -11,
    10, 0, -8, 10, 0, -5, 10, 0, -2, 10, 0, 1, 10, 0, 4, 10, 0, -16,
    8, 0, -13, 8, 0, -10, 8, 0, -7, 8, 0, -4, 8, 0, -1, 8, 0, 2, 8, 0,
    5, 8, 0, 8, 8, 0, -14, 7, 0, -11, 7, 0, -8, 7, 0, -5, 7, 0, -2, 7,
    0, 1, 7, 0, 4, 7, 0, 7, 7, 0, 10, 7, 0, -13, 5, 0, -10, 5, 0, -7,
    5, 0, -4, 5, 0, -1, 5, 0, 2, 5, 0, 5, 5, 0, 8, 5, 0, 11, 5, 0,
    -14, 4, 0, -11, 4, 0, -8, 4, 0, -5, 4, 0, -2, 4, 0, 1, 4, 0, 4, 4,
    0, 7, 4, 0, 10, 4, 0, -13, 2, 0, -10, 2, 0, -7, 2, 0, -4, 2, 0,
    -1, 2, 0, 2, 2, 0, 5, 2, 0, 8, 2, 0, 11, 2, 0, -11, 1, 0, -8, 1,
    0, -5, 1, 0, -2, 1, 0, 1, 1, 0, 4, 1, 0, 7, 1, 0, 10, 1, 0, 13, 1,
    0, -10, -1, 0, -7, -1, 0, -4, -1, 0, -1, -1, 0, 2, -1, 0, 5, -1,
    0, 8, -1, 0, 11, -1, 0, 14, -1, 0, -11, -2, 0, -8, -2, 0, -5, -2,
    0, -2, -2, 0, 1, -2, 0, 4, -2, 0, 7, -2, 0, 10, -2, 0, 13, -2, 0,
    -10, -4, 0, -7, -4, 0, -4, -4, 0, -1, -4, 0, 2, -4, 0, 5, -4, 0,
    8, -4, 0, 11, -4, 0, 14, -4, 0, -8, -5, 0, -5, -5, 0, -2, -5, 0,
    1, -5, 0, 4, -5, 0, 7, -5, 0, 10, -5, 0, 13, -5, 0, 16, -5, 0, -7,
    -7, 0, -4, -7, 0, -1, -7, 0, 2, -7, 0, 5, -7, 0, 8, -7, 0, 11, -7,
    0, 14, -7, 0, 17, -7, 0, -8, -8, 0, -5, -8, 0, -2, -8, 0, 1, -8,
    0, 4, -8, 0, 7, -8, 0, 10, -8, 0, 13, -8, 0, 16, -8, 0, -7, -10,
    0, -4, -10, 0, -1, -10, 0, 2, -10, 0, 5, -10, 0, 8, -10, 0, 11,
    -10, 0, 14, -10, 0, 17, -10, 0, -5, -11, 0, -2, -11, 0, 1, -11, 0,
    4, -11, 0, 7, -11, 0, 10, -11, 0, 13, -11, 0, 16, -11, 0, -1, -13,
    0, 2, -13, 0, 5, -13, 0, 8, -13, 0, 11, -13, 0, 14, -13, 0, 1,
    -14, 0, 4, -14, 0, 7, -14, 0, 10, -14, 0, 13, -14, 0, 5, -16, 0,
    8, -16, 0, 11, -16, 0, 7, -17, 0, 10, -17, 0]).reshape((162, 3)) / 27.0

cc162_1x1 = np.array([
    -8, -16, 0, -10, -14, 0, -7, -14, 0, -4, -14,
    0, -11, -13, 0, -8, -13, 0, -5, -13, 0, -2, -13, 0, -13, -11, 0,
    -10, -11, 0, -7, -11, 0, -4, -11, 0, -1, -11, 0, 2, -11, 0, -14,
    -10, 0, -11, -10, 0, -8, -10, 0, -5, -10, 0, -2, -10, 0, 1, -10,
    0, 4, -10, 0, -16, -8, 0, -13, -8, 0, -10, -8, 0, -7, -8, 0, -4,
    -8, 0, -1, -8, 0, 2, -8, 0, 5, -8, 0, 8, -8, 0, -14, -7, 0, -11,
    -7, 0, -8, -7, 0, -5, -7, 0, -2, -7, 0, 1, -7, 0, 4, -7, 0, 7, -7,
    0, 10, -7, 0, -13, -5, 0, -10, -5, 0, -7, -5, 0, -4, -5, 0, -1,
    -5, 0, 2, -5, 0, 5, -5, 0, 8, -5, 0, 11, -5, 0, -14, -4, 0, -11,
    -4, 0, -8, -4, 0, -5, -4, 0, -2, -4, 0, 1, -4, 0, 4, -4, 0, 7, -4,
    0, 10, -4, 0, -13, -2, 0, -10, -2, 0, -7, -2, 0, -4, -2, 0, -1,
    -2, 0, 2, -2, 0, 5, -2, 0, 8, -2, 0, 11, -2, 0, -11, -1, 0, -8,
    -1, 0, -5, -1, 0, -2, -1, 0, 1, -1, 0, 4, -1, 0, 7, -1, 0, 10, -1,
    0, 13, -1, 0, -10, 1, 0, -7, 1, 0, -4, 1, 0, -1, 1, 0, 2, 1, 0, 5,
    1, 0, 8, 1, 0, 11, 1, 0, 14, 1, 0, -11, 2, 0, -8, 2, 0, -5, 2, 0,
    -2, 2, 0, 1, 2, 0, 4, 2, 0, 7, 2, 0, 10, 2, 0, 13, 2, 0, -10, 4,
    0, -7, 4, 0, -4, 4, 0, -1, 4, 0, 2, 4, 0, 5, 4, 0, 8, 4, 0, 11, 4,
    0, 14, 4, 0, -8, 5, 0, -5, 5, 0, -2, 5, 0, 1, 5, 0, 4, 5, 0, 7, 5,
    0, 10, 5, 0, 13, 5, 0, 16, 5, 0, -7, 7, 0, -4, 7, 0, -1, 7, 0, 2,
    7, 0, 5, 7, 0, 8, 7, 0, 11, 7, 0, 14, 7, 0, 17, 7, 0, -8, 8, 0,
    -5, 8, 0, -2, 8, 0, 1, 8, 0, 4, 8, 0, 7, 8, 0, 10, 8, 0, 13, 8, 0,
    16, 8, 0, -7, 10, 0, -4, 10, 0, -1, 10, 0, 2, 10, 0, 5, 10, 0, 8,
    10, 0, 11, 10, 0, 14, 10, 0, 17, 10, 0, -5, 11, 0, -2, 11, 0, 1,
    11, 0, 4, 11, 0, 7, 11, 0, 10, 11, 0, 13, 11, 0, 16, 11, 0, -1,
    13, 0, 2, 13, 0, 5, 13, 0, 8, 13, 0, 11, 13, 0, 14, 13, 0, 1, 14,
    0, 4, 14, 0, 7, 14, 0, 10, 14, 0, 13, 14, 0, 5, 16, 0, 8, 16, 0,
    11, 16, 0, 7, 17, 0, 10, 17, 0]).reshape((162, 3)) / 27.0


# The following is a list of the critical points in the 1st Brillouin zone
# for some typical crystal structures following the conventions of Setyawan
# and Curtarolo [http://dx.doi.org/10.1016/j.commatsci.2010.05.010].
#
# In units of the reciprocal basis vectors.
#
# See http://en.wikipedia.org/wiki/Brillouin_zone
sc_special_points = {
    'cubic': {'G': [0, 0, 0],
              'M': [1 / 2, 1 / 2, 0],
              'R': [1 / 2, 1 / 2, 1 / 2],
              'X': [0, 1 / 2, 0]},
    'fcc': {'G': [0, 0, 0],
            'K': [3 / 8, 3 / 8, 3 / 4],
            'L': [1 / 2, 1 / 2, 1 / 2],
            'U': [5 / 8, 1 / 4, 5 / 8],
            'W': [1 / 2, 1 / 4, 3 / 4],
            'X': [1 / 2, 0, 1 / 2]},
    'bcc': {'G': [0, 0, 0],
            'H': [1 / 2, -1 / 2, 1 / 2],
            'P': [1 / 4, 1 / 4, 1 / 4],
            'N': [0, 0, 1 / 2]},
    'tetragonal': {'G': [0, 0, 0],
                   'A': [1 / 2, 1 / 2, 1 / 2],
                   'M': [1 / 2, 1 / 2, 0],
                   'R': [0, 1 / 2, 1 / 2],
                   'X': [0, 1 / 2, 0],
                   'Z': [0, 0, 1 / 2]},
    'orthorhombic': {'G': [0, 0, 0],
                     'R': [1 / 2, 1 / 2, 1 / 2],
                     'S': [1 / 2, 1 / 2, 0],
                     'T': [0, 1 / 2, 1 / 2],
                     'U': [1 / 2, 0, 1 / 2],
                     'X': [1 / 2, 0, 0],
                     'Y': [0, 1 / 2, 0],
                     'Z': [0, 0, 1 / 2]},
    'hexagonal': {'G': [0, 0, 0],
                  'A': [0, 0, 1 / 2],
                  'H': [1 / 3, 1 / 3, 1 / 2],
                  'K': [1 / 3, 1 / 3, 0],
                  'L': [1 / 2, 0, 1 / 2],
                  'M': [1 / 2, 0, 0]}}


# Old version of dictionary kept for backwards compatibility.
# Not for ordinary use.
ibz_points = {'cubic': {'Gamma': [0, 0, 0],
                        'X': [0, 0 / 2, 1 / 2],
                        'R': [1 / 2, 1 / 2, 1 / 2],
                        'M': [0 / 2, 1 / 2, 1 / 2]},
              'fcc': {'Gamma': [0, 0, 0],
                      'X': [1 / 2, 0, 1 / 2],
                      'W': [1 / 2, 1 / 4, 3 / 4],
                      'K': [3 / 8, 3 / 8, 3 / 4],
                      'U': [5 / 8, 1 / 4, 5 / 8],
                      'L': [1 / 2, 1 / 2, 1 / 2]},
              'bcc': {'Gamma': [0, 0, 0],
                      'H': [1 / 2, -1 / 2, 1 / 2],
                      'N': [0, 0, 1 / 2],
                      'P': [1 / 4, 1 / 4, 1 / 4]},
              'hexagonal': {'Gamma': [0, 0, 0],
                            'M': [0, 1 / 2, 0],
                            'K': [-1 / 3, 1 / 3, 0],
                            'A': [0, 0, 1 / 2],
                            'L': [0, 1 / 2, 1 / 2],
                            'H': [-1 / 3, 1 / 3, 1 / 2]},
              'tetragonal': {'Gamma': [0, 0, 0],
                             'X': [1 / 2, 0, 0],
                             'M': [1 / 2, 1 / 2, 0],
                             'Z': [0, 0, 1 / 2],
                             'R': [1 / 2, 0, 1 / 2],
                             'A': [1 / 2, 1 / 2, 1 / 2]},
              'orthorhombic': {'Gamma': [0, 0, 0],
                               'R': [1 / 2, 1 / 2, 1 / 2],
                               'S': [1 / 2, 1 / 2, 0],
                               'T': [0, 1 / 2, 1 / 2],
                               'U': [1 / 2, 0, 1 / 2],
                               'X': [1 / 2, 0, 0],
                               'Y': [0, 1 / 2, 0],
                               'Z': [0, 0, 1 / 2]}}



import numpy as np
from ase.geometry import cell_to_cellpar as c2p, cellpar_to_cell as p2c

# Make sure we get exactly zeros off-diagonal:
assert (p2c([1, 1, 1, 90, 90, 90]) == np.eye(3)).all()

eps = 2 * np.spacing(90., dtype=np.float64)


def nearly_equal(a, b):
    return np.all(np.abs(b - a) < eps)


def assert_equal(a, b):
    if not nearly_equal(a, b):
        msg = 'this:\n'
        msg += repr(a)
        msg += '\nand that:\n'
        msg += repr(b)
        msg += '\nwere supposed to be equal but are not.'
        raise AssertionError(msg)


# Constants
a = 5.43
d = a / 2.0
h = a / np.sqrt(2.0)


# Systems
# Primitive cell, non-orthorhombic, non-cubic
# Parameters
si_prim_p = np.array([h] * 3 + [60.] * 3)
# Tensor format
si_prim_m = np.array([[0., d, d],
                      [d, 0., d],
                      [d, d, 0.]])
# Tensor format in the default basis
si_prim_m2 = np.array([[2.0, 0., 0.],
                       [1.0, np.sqrt(3.0), 0.],
                       [1.0, np.sqrt(3.0) / 3.0, 2 * np.sqrt(2 / 3)]])
si_prim_m2 *= h / 2.0


# Orthorhombic cell, non-cubic
# Parameters
si_ortho_p = np.array([h] * 2 + [a] + [90.] * 3)
# Tensor format in the default basis
si_ortho_m = np.array([[h, 0.0, 0.0],
                       [0.0, h, 0.0],
                       [0.0, 0.0, a]])

# Cubic cell
# Parameters
si_cubic_p = np.array([a] * 3 + [90.] * 3)
# Tensor format in the default basis
si_cubic_m = np.array([[a, 0.0, 0.0],
                       [0.0, a, 0.0],
                       [0.0, 0.0, a]])

# Cell matrix -> cell parameters
assert_equal(c2p(si_prim_m), si_prim_p)
assert_equal(c2p(si_prim_m2), si_prim_p)
assert_equal(c2p(si_ortho_m), si_ortho_p)
assert_equal(c2p(si_cubic_m), si_cubic_p)
assert not nearly_equal(c2p(si_prim_m), si_ortho_p)

# Cell parameters -> cell matrix
assert_equal(p2c(si_prim_p), si_prim_m2)
assert_equal(p2c(si_ortho_p), si_ortho_m)
assert_equal(p2c(si_cubic_p), si_cubic_m)
assert not nearly_equal(p2c(si_prim_p), si_ortho_m)

# Idempotency (provided everything is provided in the default basis)
ref1 = si_prim_m2[:]
ref2 = si_ortho_m[:]
ref3 = si_cubic_m[:]
for i in range(20):
    ref1[:] = p2c(c2p(ref1))
    ref2[:] = p2c(c2p(ref2))
    ref3[:] = p2c(c2p(ref3))

assert_equal(ref1, si_prim_m2)
assert_equal(ref2, si_ortho_m)
assert_equal(ref3, si_cubic_m)



import matplotlib.pyplot as plt
from ase.io import iread

energies = []
distances = []

for atoms in iread('binding_curve.traj'):
    energies.append(atoms.get_potential_energy())
    distances.append(atoms.positions[1, 2] - atoms.positions[0, 2])

ax = plt.gca()
ax.plot(distances, energies)
ax.set_xlabel('Distance [Å]')
ax.set_ylabel('Total energy [eV]')
plt.show()



from gpaw import GPAW

calc = GPAW('groundstate.rutile.gpw')
atoms = calc.get_atoms()
path = calc.bandpath(density=7)
path.write('path.rutile.json')

calc.set(kpts=path, fixdensity=True,
         symmetry='off')

atoms.get_potential_energy()
bs = calc.band_structure()
bs.write('bs.rutile.json')



# Copyright (C) 2010, Jesper Friis
# (see accompanying license files for details).

"""Utility tools for atoms/geometry manipulations.
   - convenient creation of slabs and interfaces of
different orientations.
   - detection of duplicate atoms / atoms within cutoff radius
"""

import itertools
import numpy as np
from ase.geometry import complete_cell
from ase.geometry.minkowski_reduction import minkowski_reduce
from ase.utils import pbc2pbc


def translate_pretty(fractional, pbc):
    """Translates atoms such that fractional positions are minimized."""

    for i in range(3):
        if not pbc[i]:
            continue

        indices = np.argsort(fractional[:, i])
        sp = fractional[indices, i]

        widths = (np.roll(sp, 1) - sp) % 1.0
        fractional[:, i] -= sp[np.argmin(widths)]
        fractional[:, i] %= 1.0
    return fractional


def wrap_positions(positions, cell, pbc=True, center=(0.5, 0.5, 0.5),
                   pretty_translation=False, eps=1e-7):
    """Wrap positions to unit cell.

    Returns positions changed by a multiple of the unit cell vectors to
    fit inside the space spanned by these vectors.  See also the
    :meth:`ase.Atoms.wrap` method.

    Parameters:

    positions: float ndarray of shape (n, 3)
        Positions of the atoms
    cell: float ndarray of shape (3, 3)
        Unit cell vectors.
    pbc: one or 3 bool
        For each axis in the unit cell decides whether the positions
        will be moved along this axis.
    center: three float
        The positons in fractional coordinates that the new positions
        will be nearest possible to.
    pretty_translation: bool
        Translates atoms such that fractional coordinates are minimized.
    eps: float
        Small number to prevent slightly negative coordinates from being
        wrapped.

    Example:

    >>> from ase.geometry import wrap_positions
    >>> wrap_positions([[-0.1, 1.01, -0.5]],
    ...                [[1, 0, 0], [0, 1, 0], [0, 0, 4]],
    ...                pbc=[1, 1, 0])
    array([[ 0.9 ,  0.01, -0.5 ]])
    """

    if not hasattr(center, '__len__'):
        center = (center,) * 3

    pbc = pbc2pbc(pbc)
    shift = np.asarray(center) - 0.5 - eps

    # Don't change coordinates when pbc is False
    shift[np.logical_not(pbc)] = 0.0

    assert np.asarray(cell)[np.asarray(pbc)].any(axis=1).all(), (cell, pbc)

    cell = complete_cell(cell)
    fractional = np.linalg.solve(cell.T,
                                 np.asarray(positions).T).T - shift

    if pretty_translation:
        fractional = translate_pretty(fractional, pbc)
        shift = np.asarray(center) - 0.5
        shift[np.logical_not(pbc)] = 0.0
        fractional += shift
    else:
        for i, periodic in enumerate(pbc):
            if periodic:
                fractional[:, i] %= 1.0
                fractional[:, i] += shift[i]

    return np.dot(fractional, cell)


def get_layers(atoms, miller, tolerance=0.001):
    """Returns two arrays describing which layer each atom belongs
    to and the distance between the layers and origo.

    Parameters:

    miller: 3 integers
        The Miller indices of the planes. Actually, any direction
        in reciprocal space works, so if a and b are two float
        vectors spanning an atomic plane, you can get all layers
        parallel to this with miller=np.cross(a,b).
    tolerance: float
        The maximum distance in Angstrom along the plane normal for
        counting two atoms as belonging to the same plane.

    Returns:

    tags: array of integres
        Array of layer indices for each atom.
    levels: array of floats
        Array of distances in Angstrom from each layer to origo.

    Example:

    >>> import numpy as np
    >>> from ase.spacegroup import crystal
    >>> atoms = crystal('Al', [(0,0,0)], spacegroup=225, cellpar=4.05)
    >>> np.round(atoms.positions, decimals=5)
    array([[ 0.   ,  0.   ,  0.   ],
           [ 0.   ,  2.025,  2.025],
           [ 2.025,  0.   ,  2.025],
           [ 2.025,  2.025,  0.   ]])
    >>> get_layers(atoms, (0,0,1))  # doctest: +ELLIPSIS
    (array([0, 1, 1, 0]...), array([ 0.   ,  2.025]))
    """
    miller = np.asarray(miller)

    metric = np.dot(atoms.cell, atoms.cell.T)
    c = np.linalg.solve(metric.T, miller.T).T
    miller_norm = np.sqrt(np.dot(c, miller))
    d = np.dot(atoms.get_scaled_positions(), miller) / miller_norm

    keys = np.argsort(d)
    ikeys = np.argsort(keys)
    mask = np.concatenate(([True], np.diff(d[keys]) > tolerance))
    tags = np.cumsum(mask)[ikeys]
    if tags.min() == 1:
        tags -= 1

    levels = d[keys][mask]
    return tags, levels


def find_mic(v, cell, pbc=True):
    """Finds the minimum-image representation of vector(s) v"""

    pbc = cell.any(1) & pbc2pbc(pbc)
    v = np.array(v)
    single = len(v.shape) == 1
    v = np.atleast_2d(v)

    if np.sum(pbc) > 0:
        cell = complete_cell(cell)
        rcell, _ = minkowski_reduce(cell, pbc=pbc)

        # in a Minkowski-reduced cell we only need to test nearest neighbors
        cs = [np.arange(-1 * p, p + 1) for p in pbc]
        neighbor_cells = list(itertools.product(*cs))

        positions = wrap_positions(v, rcell, pbc=pbc, eps=0)
        vmin = positions.copy()
        vlen = np.linalg.norm(positions, axis=1)
        for nbr in neighbor_cells:
            trial = positions + np.dot(rcell.T, nbr)
            trial_len = np.linalg.norm(trial, axis=1)

            indices = np.where(trial_len < vlen)
            vmin[indices] = trial[indices]
            vlen[indices] = trial_len[indices]
    else:
        vmin = v.copy()
        vlen = np.linalg.norm(vmin, axis=1)

    if single:
        return vmin[0], vlen[0]
    else:
        return vmin, vlen


def get_angles(v1, v2, cell=None, pbc=None):
    """Get angles formed by two lists of vectors.

    calculate angle in degrees between vectors v1 and v2

    Set a cell and pbc to enable minimum image
    convention, otherwise angles are taken as-is.
    """

    # Check if using mic
    if cell is not None or pbc is not None:
        if cell is None or pbc is None:
            raise ValueError("cell or pbc must be both set or both be None")

        v1 = find_mic(v1, cell, pbc)[0]
        v2 = find_mic(v2, cell, pbc)[0]

    nv1 = np.linalg.norm(v1, axis=1)[:, np.newaxis]
    nv2 = np.linalg.norm(v2, axis=1)[:, np.newaxis]
    if (nv1 <= 0).any() or (nv2 <= 0).any():
        raise ZeroDivisionError('Undefined angle')
    v1 /= nv1
    v2 /= nv2

    # We just normalized the vectors, but in some cases we can get
    # bad things like 1+2e-16.  These we clip away:
    angles = np.arccos(np.einsum('ij,ij->i', v1, v2).clip(-1.0, 1.0))
    return np.degrees(angles)


def get_distances(p1, p2=None, cell=None, pbc=None):
    """Return distance matrix of every position in p1 with every position in p2

    if p2 is not set, it is assumed that distances between all positions in p1
    are desired. p2 will be set to p1 in this case.

    Use set cell and pbc to use the minimum image convention.
    """
    p1 = np.atleast_2d(p1)
    if p2 is None:
        np1 = len(p1)
        ind1, ind2 = np.triu_indices(np1, k=1)
        D = p1[ind2] - p1[ind1]
    else:
        p2 = np.atleast_2d(p2)
        D = (p2[np.newaxis, :, :] - p1[:, np.newaxis, :]).reshape((-1, 3))

    # Check if using mic
    if cell is not None or pbc is not None:
        if cell is None or pbc is None:
            raise ValueError("cell or pbc must be both set or both be None")

        D, D_len = find_mic(D, cell, pbc)
    else:
        D_len = np.sqrt((D**2).sum(1))

    if p2 is None:
        Dout = np.zeros((np1, np1, 3))
        Dout[(ind1, ind2)] = D
        Dout -= np.transpose(Dout, axes=(1, 0, 2))

        Dout_len = np.zeros((np1, np1))
        Dout_len[(ind1, ind2)] = D_len
        Dout_len += Dout_len.T
        return Dout, Dout_len

    # Expand back to matrix indexing
    D.shape = (-1, len(p2), 3)
    D_len.shape = (-1, len(p2))

    return D, D_len


def get_duplicate_atoms(atoms, cutoff=0.1, delete=False):
    """Get list of duplicate atoms and delete them if requested.

    Identify all atoms which lie within the cutoff radius of each other.
    Delete one set of them if delete == True.
    """
    from scipy.spatial.distance import pdist
    dists = pdist(atoms.get_positions(), 'sqeuclidean')
    dup = np.nonzero(dists < cutoff**2)
    rem = np.array(_row_col_from_pdist(len(atoms), dup[0]))
    if delete:
        if rem.size != 0:
            del atoms[rem[:, 0]]
    else:
        return rem


def _row_col_from_pdist(dim, i):
    """Calculate the i,j index in the square matrix for an index in a
    condensed (triangular) matrix.
    """
    i = np.array(i)
    b = 1 - 2 * dim
    x = (np.floor((-b - np.sqrt(b**2 - 8 * i)) / 2)).astype(int)
    y = (i + x * (b + x + 2) / 2 + 1).astype(int)
    if i.shape:
        return list(zip(x, y))
    else:
        return [(x, y)]


def permute_axes(atoms, permutation):
    """Permute axes of unit cell and atom positions. Considers only cell and
    atomic positions. Other vector quantities such as momenta are not
    modified."""
    assert (np.sort(permutation) == np.arange(3)).all()

    permuted = atoms.copy()
    scaled = permuted.get_scaled_positions()
    permuted.set_cell(permuted.cell.permute_axes(permutation),
                      scale_atoms=False)
    permuted.set_scaled_positions(scaled[:, permutation])
    permuted.set_pbc(permuted.pbc[permutation])
    return permuted



from ase import Atoms, Atom
from ase.calculators.vasp import Vasp

a = [6.5, 6.5, 7.7]
d = 2.3608
NaCl = Atoms([Atom('Na', [0, 0, 0], magmom=1.928),
              Atom('Cl', [0, 0, d], magmom=0.75)],
             cell=a)

calc = Vasp(prec='Accurate',
            xc='PBE',
            lreal=False)
NaCl.set_calculator(calc)

print(NaCl.get_magnetic_moment())



"""Functions that are important for the genetic algorithm.
Shorthand for setting and getting
- the raw_score
- the neighbor_list
- the parametrization
of an atoms object.
"""


def set_raw_score(atoms, raw_score):
    """Set the raw_score of an atoms object in the
    atoms.info['key_value_pairs'] dictionary.
    
    Parameters
    ----------
    atoms : Atoms object
        The atoms object that corresponds to this raw_score
    raw_score : float or int
        Independent calculation of how fit the candidate is.
    """
    if 'key_value_pairs' not in atoms.info:
        atoms.info['key_value_pairs'] = {}
    atoms.info['key_value_pairs']['raw_score'] = raw_score


def get_raw_score(atoms):
    """Gets the raw_score of the supplied atoms object.

    Parameters
    ----------
    atoms : Atoms object
        The atoms object from which the raw_score will be returned.

    Returns
    -------
    raw_score : float or int
        The raw_score set previously.
    """
    return atoms.info['key_value_pairs']['raw_score']

    
def set_parametrization(atoms, parametrization):
    if 'data' not in atoms.info:
        atoms.info['data'] = {}
    atoms.info['data']['parametrization'] = parametrization

    
def get_parametrization(atoms):
    if 'parametrization' in atoms.info['data']:
        return atoms.info['data']['parametrization']
    else:
        raise ValueError('Trying to get the parametrization before it is set!')

        
def set_neighbor_list(atoms, neighbor_list):
    if 'data' not in atoms.info:
        atoms.info['data'] = {}
    atoms.info['data']['neighborlist'] = neighbor_list

    
def get_neighbor_list(atoms):
    if 'neighborlist' in atoms.info['data']:
        return atoms.info['data']['neighborlist']
    else:
        return None



import numpy as np
import matplotlib.pyplot as plt

fig = plt.figure(1, dpi=80, figsize=(4.2, 6))
fig.subplots_adjust(left=.16, right=.97, top=.97, bottom=.05)

# Plot KS bands
k, eps = np.loadtxt('KSbands.txt', unpack=True)
plt.plot(k, eps, 'ro', label='DFT', ms=9)

# Plot Wannier bands
k, eps = np.loadtxt('WANbands.txt', unpack=True)
plt.plot(k, eps, 'k.', label='Wannier')

plt.plot([-.5, .5], [1, 1], 'k:', label='_nolegend_')
plt.text(-.5, 1, 'fixedenergy', ha='left', va='bottom')
plt.axis('tight')
plt.xticks([-.5, -.25, 0, .25, .5],
           [r'$X$', r'$\Delta$', r'$\Gamma$', r'$\Delta$', r'$X$'], size=16)
plt.ylabel(r'$E - E_F\  \rm{(eV)}$', size=16)
plt.legend()
plt.savefig('bands.png', dpi=80)
plt.show()



import re
import numpy as np

from ase.atoms import Atoms
from ase.parallel import paropen
from ase.utils import basestring
from ase.calculators.lammps import Prism, convert


def read_lammps_data(fileobj, Z_of_type=None, style="full",
                     sort_by_id=False, units="metal"):
    """Method which reads a LAMMPS data file.

    sort_by_id: Order the particles according to their id. Might be faster to
    switch it off.
    Units are set by default to the style=metal setting in LAMMPS.
    """
    if isinstance(fileobj, basestring):
        f = paropen(fileobj)
    else:
        f = fileobj

    # load everything into memory
    lines = f.readlines()

    # begin read_lammps_data
    comment = None
    N = None
    # N_types = None
    xlo = None
    xhi = None
    ylo = None
    yhi = None
    zlo = None
    zhi = None
    xy = None
    xz = None
    yz = None
    pos_in = {}
    travel_in = {}
    mol_id_in = {}
    charge_in = {}
    mass_in = {}
    vel_in = {}
    bonds_in = []
    angles_in = []
    dihedrals_in = []

    sections = [
        "Atoms",
        "Velocities",
        "Masses",
        "Charges",
        "Ellipsoids",
        "Lines",
        "Triangles",
        "Bodies",
        "Bonds",
        "Angles",
        "Dihedrals",
        "Impropers",
        "Impropers Pair Coeffs",
        "PairIJ Coeffs",
        "Pair Coeffs",
        "Bond Coeffs",
        "Angle Coeffs",
        "Dihedral Coeffs",
        "Improper Coeffs",
        "BondBond Coeffs",
        "BondAngle Coeffs",
        "MiddleBondTorsion Coeffs",
        "EndBondTorsion Coeffs",
        "AngleTorsion Coeffs",
        "AngleAngleTorsion Coeffs",
        "BondBond13 Coeffs",
        "AngleAngle Coeffs",
    ]
    header_fields = [
        "atoms",
        "bonds",
        "angles",
        "dihedrals",
        "impropers",
        "atom types",
        "bond types",
        "angle types",
        "dihedral types",
        "improper types",
        "extra bond per atom",
        "extra angle per atom",
        "extra dihedral per atom",
        "extra improper per atom",
        "extra special per atom",
        "ellipsoids",
        "lines",
        "triangles",
        "bodies",
        "xlo xhi",
        "ylo yhi",
        "zlo zhi",
        "xy xz yz",
    ]
    sections_re = "(" + "|".join(sections).replace(" ", "\\s+") + ")"
    header_fields_re = "(" + "|".join(header_fields).replace(" ", "\\s+") + ")"

    section = None
    header = True
    for line in lines:
        if comment is None:
            comment = line.rstrip()
        else:
            line = re.sub("#.*", "", line).rstrip().lstrip()
            if re.match("^\\s*$", line):  # skip blank lines
                continue

        # check for known section names
        m = re.match(sections_re, line)
        if m is not None:
            section = m.group(0).rstrip().lstrip()
            header = False
            continue

        if header:
            field = None
            val = None
            # m = re.match(header_fields_re+"\s+=\s*(.*)", line)
            # if m is not None: # got a header line
            #   field=m.group(1).lstrip().rstrip()
            #   val=m.group(2).lstrip().rstrip()
            # else: # try other format
            #   m = re.match("(.*)\s+"+header_fields_re, line)
            #   if m is not None:
            #       field = m.group(2).lstrip().rstrip()
            #       val = m.group(1).lstrip().rstrip()
            m = re.match("(.*)\\s+" + header_fields_re, line)
            if m is not None:
                field = m.group(2).lstrip().rstrip()
                val = m.group(1).lstrip().rstrip()
            if field is not None and val is not None:
                if field == "atoms":
                    N = int(val)
                # elif field == "atom types":
                #     N_types = int(val)
                elif field == "xlo xhi":
                    (xlo, xhi) = [float(x) for x in val.split()]
                elif field == "ylo yhi":
                    (ylo, yhi) = [float(x) for x in val.split()]
                elif field == "zlo zhi":
                    (zlo, zhi) = [float(x) for x in val.split()]
                elif field == "xy xz yz":
                    (xy, xz, yz) = [float(x) for x in val.split()]

        if section is not None:
            fields = line.split()
            if section == "Atoms":  # id *
                id = int(fields[0])
                if style == "full" and (len(fields) == 7 or len(fields) == 10):
                    # id mol-id type q x y z [tx ty tz]
                    pos_in[id] = (
                        int(fields[2]),
                        float(fields[4]),
                        float(fields[5]),
                        float(fields[6]),
                    )
                    mol_id_in[id] = int(fields[1])
                    charge_in[id] = float(fields[3])
                    if len(fields) == 10:
                        travel_in[id] = (
                            int(fields[7]),
                            int(fields[8]),
                            int(fields[9]),
                        )
                elif style == "atomic" and (
                        len(fields) == 5 or len(fields) == 8
                ):
                    # id type x y z [tx ty tz]
                    pos_in[id] = (
                        int(fields[1]),
                        float(fields[2]),
                        float(fields[3]),
                        float(fields[4]),
                    )
                    if len(fields) == 8:
                        travel_in[id] = (
                            int(fields[5]),
                            int(fields[6]),
                            int(fields[7]),
                        )
                elif (style in ("angle", "bond", "molecular")
                      ) and (len(fields) == 6 or len(fields) == 9):
                    # id mol-id type x y z [tx ty tz]
                    pos_in[id] = (
                        int(fields[2]),
                        float(fields[3]),
                        float(fields[4]),
                        float(fields[5]),
                    )
                    mol_id_in[id] = int(fields[1])
                    if len(fields) == 9:
                        travel_in[id] = (
                            int(fields[6]),
                            int(fields[7]),
                            int(fields[8]),
                        )
                if style == "charge" and (len(fields) == 6 or len(fields) == 9):
                    # id type q x y z [tx ty tz]
                    pos_in[id] = (
                        int(fields[1]),
                        float(fields[3]),
                        float(fields[4]),
                        float(fields[5]),
                    )
                    charge_in[id] = float(fields[2])
                    if len(fields) == 9:
                        travel_in[id] = (
                            int(fields[6]),
                            int(fields[7]),
                            int(fields[8]),
                        )
                else:
                    raise RuntimeError(
                        "Style '{}' not supported or invalid "
                        "number of fields {}"
                        "".format(style, len(fields))
                    )
            elif section == "Velocities":  # id vx vy vz
                vel_in[int(fields[0])] = (
                    float(fields[1]),
                    float(fields[2]),
                    float(fields[3]),
                )
            elif section == "Masses":
                mass_in[int(fields[0])] = float(fields[1])
            elif section == "Bonds":  # id type atom1 atom2
                bonds_in.append(
                    (int(fields[1]), int(fields[2]), int(fields[3]))
                )
            elif section == "Angles":  # id type atom1 atom2 atom3
                angles_in.append(
                    (
                        int(fields[1]),
                        int(fields[2]),
                        int(fields[3]),
                        int(fields[4]),
                    )
                )
            elif section == "Dihedrals":  # id type atom1 atom2 atom3 atom4
                dihedrals_in.append(
                    (
                        int(fields[1]),
                        int(fields[2]),
                        int(fields[3]),
                        int(fields[4]),
                        int(fields[5]),
                    )
                )

    # set cell
    cell = np.zeros((3, 3))
    cell[0, 0] = xhi - xlo
    cell[1, 1] = yhi - ylo
    cell[2, 2] = zhi - zlo
    if xy is not None:
        cell[1, 0] = xy
    if xz is not None:
        cell[2, 0] = xz
    if yz is not None:
        cell[2, 1] = yz

    # initialize arrays for per-atom quantities
    positions = np.zeros((N, 3))
    numbers = np.zeros((N), int)
    ids = np.zeros((N), int)
    types = np.zeros((N), int)
    if len(vel_in) > 0:
        velocities = np.zeros((N, 3))
    else:
        velocities = None
    if len(mass_in) > 0:
        masses = np.zeros((N))
    else:
        masses = None
    if len(mol_id_in) > 0:
        mol_id = np.zeros((N), int)
    else:
        mol_id = None
    if len(charge_in) > 0:
        charge = np.zeros((N), float)
    else:
        charge = None
    if len(travel_in) > 0:
        travel = np.zeros((N, 3), int)
    else:
        travel = None
    if len(bonds_in) > 0:
        bonds = [""] * N
    else:
        bonds = None
    if len(angles_in) > 0:
        angles = [""] * N
    else:
        angles = None
    if len(dihedrals_in) > 0:
        dihedrals = [""] * N
    else:
        dihedrals = None

    ind_of_id = {}
    # copy per-atom quantities from read-in values
    for (i, id) in enumerate(pos_in.keys()):
        # by id
        ind_of_id[id] = i
        if sort_by_id:
            ind = id - 1
        else:
            ind = i
        type = pos_in[id][0]
        positions[ind, :] = [pos_in[id][1], pos_in[id][2], pos_in[id][3]]
        if velocities is not None:
            velocities[ind, :] = [vel_in[id][0], vel_in[id][1], vel_in[id][2]]
        if travel is not None:
            travel[ind] = travel_in[id]
        if mol_id is not None:
            mol_id[ind] = mol_id_in[id]
        if charge is not None:
            charge[ind] = charge_in[id]
        ids[ind] = id
        # by type
        types[ind] = type
        if Z_of_type is None:
            numbers[ind] = type
        else:
            numbers[ind] = Z_of_type[type]
        if masses is not None:
            masses[ind] = mass_in[type]
    # convert units
    positions = convert(positions, "distance", units, "ASE")
    cell = convert(cell, "distance", units, "ASE")
    if masses is not None:
        masses = convert(masses, "mass", units, "ASE")
    if velocities is not None:
        velocities = convert(velocities, "velocity", units, "ASE")

    # create ase.Atoms
    at = Atoms(
        positions=positions,
        numbers=numbers,
        masses=masses,
        cell=cell,
        pbc=[True, True, True],
    )
    # set velocities (can't do it via constructor)
    if velocities is not None:
        at.set_velocities(velocities)
    at.arrays["id"] = ids
    at.arrays["type"] = types
    if travel is not None:
        at.arrays["travel"] = travel
    if mol_id is not None:
        at.arrays["mol-id"] = mol_id
    if charge is not None:
        at.arrays["initial_charges"] = charge
        at.arrays["mmcharges"] = charge.copy()

    if bonds is not None:
        for (type, a1, a2) in bonds_in:
            i_a1 = ind_of_id[a1]
            i_a2 = ind_of_id[a2]
            if len(bonds[i_a1]) > 0:
                bonds[i_a1] += ","
            bonds[i_a1] += "%d(%d)" % (i_a2, type)
        for i in range(len(bonds)):
            if len(bonds[i]) == 0:
                bonds[i] = "_"
        at.arrays["bonds"] = np.array(bonds)

    if angles is not None:
        for (type, a1, a2, a3) in angles_in:
            i_a1 = ind_of_id[a1]
            i_a2 = ind_of_id[a2]
            i_a3 = ind_of_id[a3]
            if len(angles[i_a2]) > 0:
                angles[i_a2] += ","
            angles[i_a2] += "%d-%d(%d)" % (i_a1, i_a3, type)
        for i in range(len(angles)):
            if len(angles[i]) == 0:
                angles[i] = "_"
        at.arrays["angles"] = np.array(angles)

    if dihedrals is not None:
        for (type, a1, a2, a3, a4) in dihedrals_in:
            i_a1 = ind_of_id[a1]
            i_a2 = ind_of_id[a2]
            i_a3 = ind_of_id[a3]
            i_a4 = ind_of_id[a4]
            if len(dihedrals[i_a1]) > 0:
                dihedrals[i_a1] += ","
            dihedrals[i_a1] += "%d-%d-%d(%d)" % (i_a2, i_a3, i_a4, type)
        for i in range(len(dihedrals)):
            if len(dihedrals[i]) == 0:
                dihedrals[i] = "_"
        at.arrays["dihedrals"] = np.array(dihedrals)

    at.info["comment"] = comment

    return at


def write_lammps_data(fileobj, atoms, specorder=None, force_skew=False,
                      prismobj=None, velocities=False, units="metal",
                      atom_style='atomic'):
    """Write atomic structure data to a LAMMPS data file."""
    if isinstance(fileobj, basestring):
        f = paropen(fileobj, "w", encoding="ascii")
        close_file = True
    else:
        # Presume fileobj acts like a fileobj
        f = fileobj
        close_file = False

    # FIXME: We should add a check here that the encoding of the file object
    #        is actually ascii once the 'encoding' attribute of IOFormat objects
    #        starts functioning in implementation (currently it doesn't do
    #         anything).

    if isinstance(atoms, list):
        if len(atoms) > 1:
            raise ValueError(
                "Can only write one configuration to a lammps data file!"
            )
        atoms = atoms[0]

    f.write("{0} (written by ASE) \n\n".format(f.name))

    symbols = atoms.get_chemical_symbols()
    n_atoms = len(symbols)
    f.write("{0} \t atoms \n".format(n_atoms))

    if specorder is None:
        # This way it is assured that LAMMPS atom types are always
        # assigned predictably according to the alphabetic order
        species = sorted(set(symbols))
    else:
        # To index elements in the LAMMPS data file
        # (indices must correspond to order in the potential file)
        species = specorder
    n_atom_types = len(species)
    f.write("{0}  atom types\n".format(n_atom_types))

    if prismobj is None:
        p = Prism(atoms.get_cell())
    else:
        p = prismobj

    # Get cell parameters and convert from ASE units to LAMMPS units
    xhi, yhi, zhi, xy, xz, yz = convert(p.get_lammps_prism(), "distance",
            "ASE", units)

    f.write("0.0 {0:23.17g}  xlo xhi\n".format(xhi))
    f.write("0.0 {0:23.17g}  ylo yhi\n".format(yhi))
    f.write("0.0 {0:23.17g}  zlo zhi\n".format(zhi))

    if force_skew or p.is_skewed():
        f.write(
            "{0:23.17g} {1:23.17g} {2:23.17g}  xy xz yz\n".format(
                xy, xz, yz
            )
        )
    f.write("\n\n")

    f.write("Atoms \n\n")
    pos = p.vector_to_lammps(atoms.get_positions(), wrap=True)

    if atom_style == 'atomic':
        for i, r in enumerate(pos):
            # Convert position from ASE units to LAMMPS units
            r = convert(r, "distance", "ASE", units)
            s = species.index(symbols[i]) + 1
            f.write(
                "{0:>6} {1:>3} {2:23.17g} {3:23.17g} {4:23.17g}\n".format(
                    *(i + 1, s) + tuple(r)
                )
            )
    elif atom_style == 'charge':
        charges = atoms.get_initial_charges()
        for i, (q, r) in enumerate(zip(charges, pos)):
            # Convert position and charge from ASE units to LAMMPS units
            r = convert(r, "distance", "ASE", units)
            q = convert(q, "charge", "ASE", units)
            s = species.index(symbols[i]) + 1
            f.write(
                "{0:>6} {1:>3} {2:>5} {3:23.17g} {4:23.17g} {5:23.17g}\n".format(
                    *(i + 1, s, q) + tuple(r)
                )
            )
    elif atom_style == 'full':
        charges = atoms.get_initial_charges()
        molecule = 1 # Assign all atoms to a single molecule
        for i, (q, r) in enumerate(zip(charges, pos)):
            # Convert position and charge from ASE units to LAMMPS units
            r = convert(r, "distance", "ASE", units)
            q = convert(q, "charge", "ASE", units)
            s = species.index(symbols[i]) + 1
            f.write(
                "{0:>6} {1:>3} {2:>3} {3:>5} {4:23.17g} {5:23.17g} {6:23.17g}\n".format(
                    *(i + 1, molecule, s, q) + tuple(r)
                )
            )
    else:
        raise NotImplementedError

    if velocities and atoms.get_velocities() is not None:
        f.write("\n\nVelocities \n\n")
        vel = p.vector_to_lammps(atoms.get_velocities())
        for i, v in enumerate(vel):
            # Convert velocity from ASE units to LAMMPS units
            v = convert(v, "velocity", "ASE", units)
            f.write(
                "{0:>6} {1:23.17g} {2:23.17g} {3:23.17g}\n".format(
                    *(i + 1,) + tuple(v)
                )
            )

    f.flush()
    if close_file:
        f.close()



import numpy as np


def distance(s1, s2, permute=True):
    """Get the distance between two structures s1 and s2.
    
    The distance is defined by the Frobenius norm of
    the spatial distance between all coordinates (see
    numpy.linalg.norm for the definition).

    permute: minimise the distance by 'permuting' same elements
    """

    s1 = s1.copy()
    s2 = s2.copy()
    for s in [s1, s2]:
        s.translate(-s.get_center_of_mass())
    s2pos = 1. * s2.get_positions()
    
    def align(struct, xaxis='x', yaxis='y'):
        """Align moments of inertia with the coordinate system."""
        Is, Vs = struct.get_moments_of_inertia(True)
        IV = list(zip(Is, Vs))
        IV.sort(key=lambda x: x[0])
        struct.rotate(IV[0][1], xaxis)
        
        Is, Vs = struct.get_moments_of_inertia(True)
        IV = list(zip(Is, Vs))
        IV.sort(key=lambda x: x[0])
        struct.rotate(IV[1][1], yaxis)

    align(s1)

    def dd(s1, s2, permute):
        if permute:
            s2 = s2.copy()
            dist = 0
            for a in s1:
                imin = None
                dmin = np.Inf
                for i, b in enumerate(s2):
                    if a.symbol == b.symbol:
                        d = np.sum((a.position - b.position)**2)
                        if d < dmin:
                            dmin = d
                            imin = i
                dist += dmin
                s2.pop(imin)
            return np.sqrt(dist)
        else:
            return np.linalg.norm(s1.get_positions() - s2.get_positions())

    dists = []
    # principles
    for x, y in zip(['x', '-x', 'x', '-x'], ['y', 'y', '-y', '-y']):
        s2.set_positions(s2pos)
        align(s2, x, y)
        dists.append(dd(s1, s2, permute))
   
    return min(dists)



import pickle
import sys

import numpy as np

from ase.gui.i18n import _
import ase.gui.ui as ui

graph_help_text = _("""\
Symbols:
<c>e</c>: total energy
<c>epot</c>: potential energy
<c>ekin</c>: kinetic energy
<c>fmax</c>: maximum force
<c>fave</c>: average force
<c>R[n,0-2]</c>: position of atom number <c>n</c>
<c>d(n<sub>1</sub>,n<sub>2</sub>)</c>: distance between two atoms \
<c>n<sub>1</sub></c> and <c>n<sub>2</sub></c>
<c>i</c>: current image number
<c>E[i]</c>: energy of image number <c>i</c>
<c>F[n,0-2]</c>: force on atom number <c>n</c>
<c>V[n,0-2]</c>: velocity of atom number <c>n</c>
<c>M[n]</c>: magnetic moment of atom number <c>n</c>
<c>A[0-2,0-2]</c>: unit-cell basis vectors
<c>s</c>: path length
<c>a(n1,n2,n3)</c>: angle between atoms <c>n<sub>1</sub></c>, \
<c>n<sub>2</sub></c> and <c>n<sub>3</sub></c>, centered on <c>n<sub>2</sub></c>
<c>dih(n1,n2,n3,n4)</c>: dihedral angle between <c>n<sub>1</sub></c>, \
<c>n<sub>2</sub></c>, <c>n<sub>3</sub></c> and <c>n<sub>4</sub></c>
<c>T</c>: temperature (K)\
""")


class Graphs:
    def __init__(self, gui):
        win = ui.Window('Graphs')
        self.expr = ui.Entry('', 50, self.plot)
        win.add([self.expr, ui.helpbutton(graph_help_text)])

        win.add([ui.Button(_('Plot'), self.plot, 'xy'),
                 ' x, y1, y2, ...'], 'w')
        win.add([ui.Button(_('Plot'), self.plot, 'y'),
                 ' y1, y2, ...'], 'w')
        win.add([ui.Button(_('Save'), self.save)], 'w')

        self.gui = gui

    def plot(self, type=None, expr=None, ignore_if_nan=False):
        if expr is None:
            expr = self.expr.value
        else:
            self.expr.value = expr

        try:
            data = self.gui.images.graph(expr)
        except Exception as ex:
            ui.error(ex)
            return

        if ignore_if_nan and len(data) == 2 and np.isnan(data[1]).all():
            return
        pickledata = (data, self.gui.frame, expr, type)
        self.gui.pipe('graph', pickledata)

    def save(self):
        dialog = ui.SaveFileDialog(self.gui.window.win,
                                   _('Save data to file ... '))
        filename = dialog.go()
        if filename:
            expr = self.expr.value
            data = self.gui.images.graph(expr)
            np.savetxt(filename, data.T, header=expr)


def make_plot(data, i, expr, type, show=True):
    import matplotlib.pyplot as plt
    basesize = 4
    plt.figure(figsize=(basesize * 2.5**0.5, basesize))
    m = len(data)

    if type is None:
        if m == 1:
            type = 'y'
        else:
            type = 'xy'

    if type == 'y':
        for j in range(m):
            plt.plot(data[j])
            plt.plot([i], [data[j, i]], 'o')
    else:
        for j in range(1, m):
            plt.plot(data[0], data[j])
            plt.plot([data[0, i]], [data[j, i]], 'o')
    plt.title(expr)
    if show:
        plt.show()


if __name__ == '__main__':
    make_plot(*pickle.load(sys.stdin.buffer))



import numpy as np
from ase.build import bulk, make_supercell
from ase.lattice import FCC, BCC
from ase.calculators.emt import EMT

a = 4.1
atoms = bulk('Au', a=a)
assert atoms.cell.get_bravais_lattice().name == 'FCC'

# Since FCC and BCC are reciprocal, their product is cubic:
P = BCC(2.0).tocell()
assert np.allclose(np.linalg.det(P), 4)

cubatoms = make_supercell(atoms, P)
assert np.allclose(cubatoms.cell, a * np.eye(3))

# Also test some of the Cell object methods now that we are at it:
assert len(cubatoms) == 4
assert cubatoms.cell.orthorhombic
assert np.allclose(cubatoms.cell.lengths(), a)
assert cubatoms.cell.get_bravais_lattice().name == 'CUB'


# Now check BCC --> CUB transformation:
bcc = bulk('Fe', a=a)
P = FCC(2.0).tocell()
assert np.allclose(np.linalg.det(P), 2)
cubatoms = make_supercell(bcc, P)
assert np.allclose(cubatoms.cell, a * np.eye(3))

# Finally we want to check some random transformations.
# This will produce some weird supercells, all of which should have
# the same energy.

def getenergy(atoms, eref=None):
    atoms.calc = EMT()
    e = atoms.get_potential_energy() / len(atoms)
    if eref is not None:
        err = abs(e - eref)
        print('natoms', len(atoms), 'err', err)
        assert err < 1e-12, err
    return e

rng = np.random.RandomState(44)

e0 = getenergy(atoms)

imgs = []

i = 0
while i < 10:
    P = rng.randint(-2, 3, size=(3, 3))
    detP = np.linalg.det(P)
    if detP == 0:
        continue
    elif detP < 0:
        P[0] *= -1
    bigatoms = make_supercell(atoms, P)
    imgs.append(bigatoms)
    getenergy(bigatoms, eref=e0)
    i += 1

# from ase.visualize import view
# view(imgs)



from ase.calculators.siesta import Siesta
from ase.calculators.siesta.parameters import Species, PAOBasisBlock
from ase.optimize import QuasiNewton
from ase import Atoms

atoms = Atoms(
    '3H',
    [(0.0, 0.0, 0.0),
     (0.0, 0.0, 0.5),
     (0.0, 0.0, 1.0)],
    cell=[10, 10, 10])

basis_set = PAOBasisBlock(
    """1
0  2 S 0.2
0.0 0.0""")
atoms.set_tags([0, 1, 0])
siesta = Siesta(
    species=[
        Species(symbol='H', tag=None, basis_set='SZ'),
        Species(symbol='H', tag=1, basis_set=basis_set, ghost=True)])

atoms.set_calculator(siesta)
dyn = QuasiNewton(atoms, trajectory='h.traj')
dyn.run(fmax=0.02)



"""Check density of states tetrahedron code."""
import numpy as np
from ase.dft.dos import linear_tetrahedron_integration as lti
from ase.dft.kpoints import monkhorst_pack

cell = np.eye(3)
shape = (11, 13, 9)
kpts = np.dot(monkhorst_pack(shape),
              np.linalg.inv(cell).T).reshape(shape + (3,))

# Free electron eigenvalues:
eigs = 0.5 * (kpts**2).sum(3)[..., np.newaxis]  # new axis for 1 band

energies = np.linspace(0.0001, eigs.max() + 0.0001, 500)

# Do 3-d, 2-d and 1-d:
dos3 = lti(cell, eigs, energies)
eigs = eigs[:, :, 4:5]
dos2 = lti(cell, eigs, energies)
eigs = eigs[5:6]
dos1 = lti(cell, eigs, energies)

# With weights:
dos1w = lti(cell, eigs, energies, np.ones_like(eigs))
assert abs(dos1 - dos1w).max() < 2e-14

# Analytic results:
ref3 = 4 * np.pi * (2 * energies)**0.5
ref2 = 2 * np.pi * np.ones_like(energies)
ref1 = 2 * (2 * energies)**-0.5

mask = np.bitwise_and(energies > 0.02, energies < 0.1)
dims = 1
for dos, ref in [(dos1, ref1), (dos2, ref2), (dos3, ref3)]:
    error = abs(1 - dos / ref)[mask].max()
    norm = dos.sum() * (energies[1] - energies[0])
    print(dims, norm, error)
    assert error < 0.2, error
    assert abs(norm - 1) < 0.11**dims, norm
    if 0:
        import matplotlib.pyplot as plt
        plt.plot(energies, dos)
        plt.plot(energies, ref)
        plt.show()
    dims += 1



from ase.db import connect
from ase import Atoms


# Data for a plot:
plot = {'a': [0, 1, 2],
        'b': [1.2, 1.1, 1.0],
        'abplot': {'title': 'Example',
                   'data': [{'x': 'a',
                             'y': 'b',
                             'label': 'label1',
                             'style': 'o-g'}],
                   'xlabel': 'blah-blah [eV]',
                   'ylabel': 'Answers'}}

for name in ['md.json', 'md.db']:
    print(name)
    db = connect(name)
    db.write(Atoms('H'), answer=42, kind='atom', foo=True)
    db.write(Atoms('H2O'), answer=117, kind='molecule', foo=False, data=plot)
    db.metadata = {'test': 'ok'}
    db.metadata = {
        'default_columns': ['formula', 'answer', 'kind'],
        'special_keys': [
            ('SELECT', 'kind'),
            ('BOOL', 'foo'),
            ('RANGE', 'ans', 'Answer', [('ANS', 'answer')])],
        'key_descriptions': {
            'kind': ('Type', 'Type of system', ''),
            'answer': ('Answer', 'Answer to question', 'eV')}}

    db = connect(name)
    md = db.metadata
    assert 'formula' in md['default_columns']
    md['title'] = 'TEST'
    db.metadata = md

    assert connect(name).metadata['title'] == 'TEST'



import numpy as np

from ase.calculators.singlepoint import SinglePointCalculator
from ase.atom import Atom
from ase.atoms import Atoms
from ase.utils import basestring


def read_dacapo_text(fileobj):
    if isinstance(fileobj, basestring):
        fileobj = open(fileobj)

    lines = fileobj.readlines()
    i = lines.index(' Structure:             A1           A2            A3\n')
    cell = np.array([[float(w) for w in line.split()[2:5]]
                     for line in lines[i + 1:i + 4]]).transpose()
    i = lines.index(' Structure:  >>         Ionic positions/velocities ' +
                    'in cartesian coordinates       <<\n')
    atoms = []
    for line in lines[i + 4:]:
        words = line.split()
        if len(words) != 9:
            break
        Z, x, y, z = words[2:6]
        atoms.append(Atom(int(Z), [float(x), float(y), float(z)]))

    atoms = Atoms(atoms, cell=cell.tolist())

    try:
        i = lines.index(
            ' DFT:  CPU time                           Total energy\n')
    except ValueError:
        pass
    else:
        column = lines[i + 3].split().index('selfcons') - 1
        try:
            i2 = lines.index(' ANALYSIS PART OF CODE\n', i)
        except ValueError:
            pass
        else:
            while i2 > i:
                if lines[i2].startswith(' DFT:'):
                    break
                i2 -= 1
            energy = float(lines[i2].split()[column])
            atoms.set_calculator(SinglePointCalculator(atoms, energy=energy))

    return atoms



def read_dacapo(filename):
    from ase.io.pupynere import NetCDFFile

    nc = NetCDFFile(filename)
    vars = nc.variables

    cell = vars['UnitCell'][-1]
    try:
        magmoms = vars['InitialAtomicMagneticMoment'][:]
    except KeyError:
        magmoms = None
    try:
        tags = vars['AtomTags'][:]
    except KeyError:
        tags = None
    atoms = Atoms(scaled_positions=vars['DynamicAtomPositions'][-1],
                  symbols=[(a + b).strip()
                           for a, b in vars['DynamicAtomSpecies'][:]],
                  cell=cell,
                  magmoms=magmoms,
                  tags=tags,
                  pbc=True)

    try:
        energy = vars['TotalEnergy'][-1]
        force = vars['DynamicAtomForces'][-1]
    except KeyError:
        energy = None
        force = None
    calc = SinglePointCalculator(atoms, energy=energy, forces=force)  ### Fixme magmoms
    atoms.set_calculator(calc)
        
    return atoms



"""LAMMPS has the options to use several internal units (which can be different
from the ones used in ase).  Mapping is therefore necessary.

See: https://lammps.sandia.gov/doc/units.html
 """

from ase import units
from . import unitconvert_constants as u

# !TODO add reduced Lennard-Jones units?

# NOTE: We assume a three-dimensional simulation here!
DIM = 3.0

UNITSETS = {}

UNITSETS["ASE"] = dict(
    mass=1.0 / units.kg,
    distance=1.0 / units.m,
    time=1.0 / units.second,
    energy=1.0 / units.J,
    velocity=units.second / units.m,
    force=units.m / units.J,
    pressure=1.0 / units.Pascal,
    charge=1.0 / units.C,
)

UNITSETS["real"] = dict(
    mass=u.gram_per_mole_si,
    distance=u.angstrom_si,
    time=u.femtosecond_si,
    energy=u.kcal_per_mole_si,
    velocity=u.angstrom_per_femtosecond_si,
    force=u.kcal_per_mole_angstrom_si,
    torque=u.kcal_per_mole_si,
    temperature=u.kelvin_si,
    pressure=u.atmosphere_si,
    dynamic_viscosity=u.poise_si,
    charge=u.e_si,
    dipole=u.electron_angstrom_si,
    electric_field=u.volt_per_angstrom_si,
    density=u.gram_si / u.centimeter_si ** DIM,
)

UNITSETS["metal"] = dict(
    mass=u.gram_per_mole_si,
    distance=u.angstrom_si,
    time=u.picosecond_si,
    energy=u.ev_si,
    velocity=u.angstrom_per_picosecond_si,
    force=u.ev_per_angstrom_si,
    torque=u.ev_si,
    temperature=u.kelvin_si,
    pressure=u.bar_si,
    dynamic_viscosity=u.poise_si,
    charge=u.e_si,
    dipole=u.electron_angstrom_si,
    electric_field=u.volt_per_angstrom_si,
    density=u.gram_si / u.centimeter_si ** DIM,
)

UNITSETS["si"] = dict(
    mass=u.kilogram_si,
    distance=u.meter_si,
    time=u.second_si,
    energy=u.joule_si,
    velocity=u.meter_per_second_si,
    force=u.newton_si,
    torque=u.joule_si,
    temperature=u.kelvin_si,
    pressure=u.pascal_si,
    dynamic_viscosity=u.pascal_si * u.second_si,
    charge=u.coulomb_si,
    dipole=u.coulomb_meter_si,
    electric_field=u.volt_per_meter_si,
    density=u.kilogram_si / u.meter_si ** DIM,
)

UNITSETS["cgs"] = dict(
    mass=u.gram_si,
    distance=u.centimeter_si,
    time=u.second_si,
    energy=u.erg_si,
    velocity=u.centimeter_per_second_si,
    force=u.dyne_si,
    torque=u.dyne_centimeter_si,
    temperature=u.kelvin_si,
    pressure=u.dyne_per_centimetersq_si,  # or barye =u. 1.0e-6 bars
    dynamic_viscosity=u.poise_si,
    charge=u.statcoulomb_si,  # or esu (4.8032044e-10 is a proton)
    dipole=u.statcoulomb_centimeter_si,  # =u. 10^18 debye,
    electric_field=u.statvolt_per_centimeter_si,  # or dyne / esu
    density=u.gram_si / (u.centimeter_si ** DIM),
)

UNITSETS["electron"] = dict(
    mass=u.amu_si,
    distance=u.bohr_si,
    time=u.femtosecond_si,
    energy=u.hartree_si,
    velocity=u.bohr_per_atu_si,
    force=u.hartree_per_bohr_si,
    temperature=u.kelvin_si,
    pressure=u.pascal_si,
    charge=u.e_si,  # multiple of electron charge (1.0 is a proton)
    dipole=u.debye_si,
    electric_field=u.volt_per_centimeter_si,
)

UNITSETS["micro"] = dict(
    mass=u.picogram_si,
    distance=u.micrometer_si,
    time=u.microsecond_si,
    energy=u.picogram_micrometersq_per_microsecondsq_si,
    velocity=u.micrometer_per_microsecond_si,
    force=u.picogram_micrometer_per_microsecondsq_si,
    torque=u.picogram_micrometersq_per_microsecondsq_si,
    temperature=u.kelvin_si,
    pressure=u.picogram_per_micrometer_microsecondsq_si,
    dynamic_viscosity=u.picogram_per_micrometer_microsecond_si,
    charge=u.picocoulomb_si,  # (1.6021765e-7 is a proton),
    dipole=u.picocoulomb_micrometer_si,
    electric_field=u.volt_per_micrometer_si,
    density=u.picogram_si / (u.micrometer_si) ** DIM,
)

UNITSETS["nano"] = dict(
    mass=u.attogram_si,
    distance=u.nanometer_si,
    time=u.nanosecond_si,
    energy=u.attogram_nanometersq_per_nanosecondsq_si,
    velocity=u.nanometer_per_nanosecond_si,
    force=u.attogram_nanometer_per_nanosecondsq_si,
    torque=u.attogram_nanometersq_per_nanosecondsq_si,
    temperature=u.kelvin_si,
    pressure=u.attogram_per_nanometer_nanosecondsq_si,
    dynamic_viscosity=u.attogram_per_nanometer_nanosecond_si,
    charge=u.e_si,  # multiple of electron charge (1.0 is a proton)
    dipole=u.electron_nanometer_si,
    electric_field=u.volt_si / u.nanometer_si,
    density=u.attogram_si / u.nanometer_si ** DIM,
)


def convert(value, quantity, fromunits, tounits):
    """Convert units between LAMMPS and ASE.

    :param value: converted value
    :param quantity: mass, distance, time, energy, velocity, force, torque,
    temperature, pressure, dynamic_viscosity, charge, dipole,
    electric_field or density
    :param fromunits: ASE, metal, real or other (see lammps docs).
    :param tounits: ASE, metal, real or other
    :returns: converted value
    :rtype:
    """
    return UNITSETS[fromunits][quantity] / UNITSETS[tounits][quantity] * value



from ase.io import iread, write
from ase.build import bulk
from ase.collections import g2
import io

images = [bulk('Si') + bulk('Fe')] + list(g2)

buf = io.BytesIO()
write(buf, images, format='traj')
txt = buf.getvalue()

buf = io.BytesIO()
buf.write(txt)
buf.seek(0)

images2 = list(iread(buf, format='traj'))

for atoms1, atoms2 in zip(images, images2):
    assert atoms1 == atoms2



import numpy as np
from gpaw import GPAW, PW

from ase.calculators.dftd3 import DFTD3
from ase.build import bulk
from ase.constraints import UnitCellFilter

from ase.optimize import LBFGS

np.random.seed(0)

diamond = bulk('C')
diamond.rattle(stdev=0.1, seed=0)
diamond.cell += np.random.normal(scale=0.1, size=(3,3))
dft = GPAW(xc='PBE', kpts=(8,8,8), mode=PW(400))
d3 = DFTD3(dft=dft)
diamond.set_calculator(d3)

ucf = UnitCellFilter(diamond)

opt = LBFGS(ucf, logfile='diamond_opt.log', trajectory='diamond_opt.traj')
opt.run(fmax=0.05)



"""
Run some VASP tests to ensure that the VASP calculator works. This
is conditional on the existence of the VASP_COMMAND or VASP_SCRIPT
environment variables

"""

from ase.test.vasp import installed2 as installed

assert installed()

from ase import Atoms
from ase.io import write
from ase.calculators.vasp import Vasp2 as Vasp
import numpy as np

def array_almost_equal(a1, a2, tol=np.finfo(type(1.0)).eps):
    """Replacement for old numpy.testing.utils.array_almost_equal."""
    return (np.abs(a1 - a2) < tol).all()

d = 1.14
co = Atoms('CO', positions=[(0, 0, 0), (0, 0, d)],
              pbc=True)
co.center(vacuum=5.)

calc = Vasp(
            xc = 'PBE',
            prec = 'Low',
            algo = 'Fast',
            ismear= 0,
            sigma = 1.,
            istart = 0,
            lwave = False,
            lcharg = False)

co.set_calculator(calc)
en = co.get_potential_energy()
write('vasp_co.traj', co)
assert abs(en + 14.918933) < 5e-3

# Secondly, check that restart from the previously created VASP output works

calc2 = Vasp(restart=True)
co2 = calc2.get_atoms()

# Need tolerance of 1e-14 because VASP itself changes coordinates
# slightly between reading POSCAR and writing CONTCAR even if no ionic
# steps are made.
assert array_almost_equal(co.positions, co2.positions, 1e-14)

assert en - co2.get_potential_energy() == 0.
assert array_almost_equal(calc.get_stress(co), calc2.get_stress(co2))
assert array_almost_equal(calc.get_forces(co), calc2.get_forces(co2))
assert array_almost_equal(calc.get_eigenvalues(), calc2.get_eigenvalues())
assert calc.get_number_of_bands() == calc2.get_number_of_bands()
assert calc.get_xc_functional() == calc2.get_xc_functional()

# Cleanup
calc.clean()



import time
from distutils.version import LooseVersion
from ase.io.utils import PlottingVariables, make_patch_list


class EPS(PlottingVariables):
    def __init__(self, atoms,
                 rotation='', radii=None,
                 bbox=None, colors=None, scale=20, maxwidth=500,
                 **kwargs):
        """Encapsulated PostScript writer.

        show_unit_cell: int
            0: Don't show unit cell (default).  1: Show unit cell.
            2: Show unit cell and make sure all of it is visible.
        """
        PlottingVariables.__init__(
            self, atoms, rotation=rotation,
            radii=radii, bbox=bbox, colors=colors, scale=scale,
            maxwidth=maxwidth,
            **kwargs)

    def write(self, filename):
        self.filename = filename
        self.write_header()
        self.write_body()
        self.write_trailer()

    def write_header(self):
        import matplotlib
        if LooseVersion(matplotlib.__version__) <= '0.8':
            raise RuntimeError('Your version of matplotlib (%s) is too old' %
                               matplotlib.__version__)

        from matplotlib.backends.backend_ps import RendererPS, psDefs

        self.fd = open(self.filename, 'w')
        self.fd.write('%!PS-Adobe-3.0 EPSF-3.0\n')
        self.fd.write('%%Creator: G2\n')
        self.fd.write('%%CreationDate: %s\n' % time.ctime(time.time()))
        self.fd.write('%%Orientation: portrait\n')
        bbox = (0, 0, self.w, self.h)
        self.fd.write('%%%%BoundingBox: %d %d %d %d\n' % bbox)
        self.fd.write('%%EndComments\n')

        Ndict = len(psDefs)
        self.fd.write('%%BeginProlog\n')
        self.fd.write('/mpldict %d dict def\n' % Ndict)
        self.fd.write('mpldict begin\n')
        for d in psDefs:
            d = d.strip()
            for l in d.split('\n'):
                self.fd.write(l.strip() + '\n')
        self.fd.write('%%EndProlog\n')

        self.fd.write('mpldict begin\n')
        self.fd.write('%d %d 0 0 clipbox\n' % (self.w, self.h))

        self.renderer = RendererPS(self.w, self.h, self.fd)

    def write_body(self):
        patch_list = make_patch_list(self)
        for patch in patch_list:
            patch.draw(self.renderer)

    def write_trailer(self):
        self.fd.write('end\n')
        self.fd.write('showpage\n')
        self.fd.close()


def write_eps(filename, atoms, **parameters):
    if isinstance(atoms, list):
        assert len(atoms) == 1
        atoms = atoms[0]
    EPS(atoms, **parameters).write(filename)
